@@ -3,19 +3,21 @@
  *
  * Copyright (C) 2004  Southern Storm Software, Pty Ltd.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This file is part of the libjit library.
  *
- * This program is distributed in the hope that it will be useful,
+ * The libjit library is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation, either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * The libjit library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with the libjit library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "jit-internal.h"
@@ -67,13 +69,13 @@
 @*/
 
 /*@
- * @deftypefun {void *} jit_malloc ({unsigned int} size)
- * Allocate @code{size} bytes of memory from the heap.
+ * @deftypefun {void *} jit_malloc (unsigned int @var{size})
+ * Allocate @var{size} bytes of memory from the heap.
  * @end deftypefun
  *
- * @deftypefun {type *} jit_new (type)
- * Allocate @code{sizeof(type)} bytes of memory from the heap and
- * cast the return pointer to @code{type *}.  This is a macro that
+ * @deftypefun {type *} jit_new (@var{type})
+ * Allocate @code{sizeof(@var{type})} bytes of memory from the heap and
+ * cast the return pointer to @code{@var{type} *}.  This is a macro that
  * wraps up the underlying @code{jit_malloc} function and is less
  * error-prone when allocating structures.
  * @end deftypefun
@@ -84,14 +86,14 @@ void *jit_malloc(unsigned int size)
 }
 
 /*@
- * @deftypefun {void *} jit_calloc ({unsigned int} num, {unsigned int} size)
- * Allocate @code{num * size} bytes of memory from the heap and clear
+ * @deftypefun {void *} jit_calloc (unsigned int @var{num}, unsigned int @var{size})
+ * Allocate @code{@var{num} * @var{size}} bytes of memory from the heap and clear
  * them to zero.
  * @end deftypefun
  *
- * @deftypefun {type *} jit_cnew (type)
- * Allocate @code{sizeof(type)} bytes of memory from the heap and
- * cast the return pointer to @code{type *}.  The memory is cleared
+ * @deftypefun {type *} jit_cnew (@var{type})
+ * Allocate @code{sizeof(@var{type})} bytes of memory from the heap and
+ * cast the return pointer to @code{@var{type} *}.  The memory is cleared
  * to zero.
  * @end deftypefun
 @*/
@@ -101,9 +103,9 @@ void *jit_calloc(unsigned int num, unsigned int size)
 }
 
 /*@
- * @deftypefun {void *} jit_realloc ({void *} ptr, {unsigned int} size)
- * Re-allocate the memory at @code{ptr} to be @code{size} bytes in size.
- * The memory block at @code{ptr} must have been allocated by a previous
+ * @deftypefun {void *} jit_realloc (void *@var{ptr}, unsigned int @var{size})
+ * Re-allocate the memory at @var{ptr} to be @var{size} bytes in size.
+ * The memory block at @var{ptr} must have been allocated by a previous
  * call to @code{jit_malloc}, @code{jit_calloc}, or @code{jit_realloc}.
  * @end deftypefun
 @*/
@@ -113,8 +115,8 @@ void *jit_realloc(void *ptr, unsigned int size)
 }
 
 /*@
- * @deftypefun void jit_free ({void *} ptr)
- * Free the memory at @code{ptr}.  It is safe to pass a NULL pointer.
+ * @deftypefun void jit_free (void *@var{ptr})
+ * Free the memory at @var{ptr}.  It is safe to pass a NULL pointer.
  * @end deftypefun
 @*/
 void jit_free(void *ptr)
@@ -126,7 +128,7 @@ void jit_free(void *ptr)
 }
 
 /*@
- * @deftypefun {void *} jit_malloc_exec ({unsigned int} size)
+ * @deftypefun {void *} jit_malloc_exec (unsigned int @var{size})
  * Allocate a block of memory that is read/write/executable.  Such blocks
  * are used to store JIT'ed code, function closures, and other trampolines.
  * The size should be a multiple of @code{jit_exec_page_size()}.
@@ -141,10 +143,14 @@ void jit_free(void *ptr)
 @*/
 void *jit_malloc_exec(unsigned int size)
 {
-#ifdef JIT_USE_MMAP
-	void *ptr = mmap(0, size, PROT_READ | PROT_WRITE | PROT_EXEC,
-							  MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
-
+#if defined(JIT_WIN32_PLATFORM)
+	return VirtualAlloc(NULL, size,
+			    MEM_COMMIT | MEM_RESERVE,
+			    PAGE_EXECUTE_READWRITE);
+#elif defined(JIT_USE_MMAP)
+	void *ptr = mmap(0, size,
+			 PROT_READ | PROT_WRITE | PROT_EXEC,
+			 MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if(ptr == (void *)-1)
 	{
 		return (void *)0;
@@ -156,9 +162,9 @@ void *jit_malloc_exec(unsigned int size)
 }
 
 /*@
- * @deftypefun void jit_free_exec ({void *} ptr, {unsigned int} size)
+ * @deftypefun void jit_free_exec (void *@var{ptr}, unsigned int @var{size})
  * Free a block of memory that was previously allocated by
- * @code{jit_malloc_exec}.  The @code{size} must be identical to the
+ * @code{jit_malloc_exec}.  The @var{size} must be identical to the
  * original allocated size, as some systems need to know this information
  * to be able to free the block.
  * @end deftypefun
@@ -167,17 +173,19 @@ void jit_free_exec(void *ptr, unsigned int size)
 {
 	if(ptr)
 	{
-	#ifdef JIT_USE_MMAP
+#if defined(JIT_WIN32_PLATFORM)
+		VirtualFree(ptr, 0, MEM_RELEASE);
+#elif defined(JIT_USE_MMAP)
 		munmap(ptr, size);
-	#else
+#else
 		free(ptr);
-	#endif
+#endif
 	}
 }
 
 /*@
- * @deftypefun void jit_flush_exec ({void *} ptr, {unsigned int} size)
- * Flush the contents of the block at @code{ptr} from the CPU's
+ * @deftypefun void jit_flush_exec (void *@var{ptr}, unsigned int @var{size})
+ * Flush the contents of the block at @var{ptr} from the CPU's
  * data and instruction caches.  This must be used after the code is
  * written to an executable code segment, but before the code is
  * executed, to prepare it for execution.
