@@ -3,19 +3,21 @@
  *
  * Copyright (C) 2004  Southern Storm Software, Pty Ltd.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This file is part of the libjit library.
  *
- * This program is distributed in the hope that it will be useful,
+ * The libjit library is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation, either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * The libjit library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with the libjit library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #ifndef	_JIT_APPLY_FUNC_H
@@ -57,11 +59,26 @@
 			__builtin_return((return_buf)); \
 		} while (0)
 
+#define	jit_builtin_return_long(return_buf)	\
+		do { \
+			__builtin_return((return_buf)); \
+		} while (0)
+
 #define	jit_builtin_return_float(return_buf)	\
 		do { \
 			__builtin_return((return_buf)); \
 		} while (0)
 
+#define	jit_builtin_return_double(return_buf)	\
+		do { \
+			__builtin_return((return_buf)); \
+		} while (0)
+
+#define	jit_builtin_return_nfloat(return_buf)	\
+		do { \
+			__builtin_return((return_buf)); \
+		} while (0)
+
 #endif
 
 /*
