@@ -3,19 +3,21 @@
  *
  * Copyright (C) 2004  Southern Storm Software, Pty Ltd.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This file is part of the libjit library.
  *
- * This program is distributed in the hope that it will be useful,
+ * The libjit library is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation, either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * The libjit library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with the libjit library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "jit-internal.h"
@@ -23,6 +25,9 @@
 #include "jit-apply-rules.h"
 #include "jit-apply-func.h"
 #include "jit-cache.h"
+#if HAVE_STDLIB_H
+	#include <stdlib.h>
+#endif
 #if HAVE_ALLOCA_H
 	#include <alloca.h>
 #endif
@@ -59,6 +64,14 @@ performed with @code{jit_closure_create}.
 
 @*/
 
+typedef enum
+{
+	_JIT_APPLY_RETURN_TYPE_OTHER = 0,
+	_JIT_APPLY_RETURN_TYPE_FLOAT32 = 1,
+	_JIT_APPLY_RETURN_TYPE_FLOAT64 = 2,
+	_JIT_APPLY_RETURN_TYPE_NFLOAT = 3
+} _jit_apply_return_type;
+
 /*
  * Flags that indicate which structure sizes are returned in registers.
  */
@@ -256,9 +269,13 @@ static void jit_apply_builder_add_arguments
 			case JIT_TYPE_STRUCT:
 			case JIT_TYPE_UNION:
 			{
-				jit_apply_builder_add_struct
-					(builder, args[param], jit_type_get_size(type),
-					 jit_type_get_alignment(type));
+#ifdef HAVE_JIT_BUILTIN_APPLY_STRUCT
+				_jit_builtin_apply_add_struct(builder, args[param], type);
+#else
+				jit_apply_builder_add_struct(builder, args[param],
+							     jit_type_get_size(type),
+							     jit_type_get_alignment(type));
+#endif
 			}
 			break;
 		}
@@ -370,21 +387,24 @@ static void jit_apply_builder_get_return
 		case JIT_TYPE_STRUCT:
 		case JIT_TYPE_UNION:
 		{
+#ifdef HAVE_JIT_BUILTIN_APPLY_STRUCT_RETURN
+			_jit_builtin_apply_get_struct_return(builder, return_value, result, type);
+#else
 			unsigned int size = jit_type_get_size(type);
-			jit_apply_builder_get_struct_return
-				(builder, size, return_value, result);
+			jit_apply_builder_get_struct_return(builder, size, return_value, result);
+#endif
 		}
 		break;
 	}
 }
 
 /*@
- * @deftypefun void jit_apply (jit_type_t signature, {void *} func, {void **} args, {unsigned int} num_fixed_args, {void *} return_value)
+ * @deftypefun void jit_apply (jit_type_t signature, void *@var{func}, void **@var{args}, unsigned int @var{num_fixed_args}, void *@var{return_value})
  * Call a function that has a particular function signature.
- * If the signature has more than @code{num_fixed_args} arguments,
+ * If the signature has more than @var{num_fixed_args} arguments,
  * then it is assumed to be a vararg call, with the additional
  * arguments passed in the vararg argument area on the stack.
- * The @code{signature} must specify the type of all arguments,
+ * The @var{signature} must specify the type of all arguments,
  * including those in the vararg argument area.
  * @end deftypefun
 @*/
@@ -437,10 +457,10 @@ void jit_apply(jit_type_t signature, void *func,
 }
 
 /*@
- * @deftypefun void jit_apply_raw (jit_type_t signature, {void *} func, {void *} args, {void *} return_value)
+ * @deftypefun void jit_apply_raw (jit_type_t @var{signature}, void *@var{func}, void *@var{args}, void *@var{return_value})
  * Call a function, passing a set of raw arguments.  This can only
  * be used if @code{jit_raw_supported} returns non-zero for the signature.
- * The @code{args} value is assumed to be an array of @code{jit_nint} values
+ * The @var{args} value is assumed to be an array of @code{jit_nint} values
  * that correspond to each of the arguments.  Raw function calls
  * are slightly faster than their non-raw counterparts, but can
  * only be used in certain circumstances.
@@ -474,7 +494,7 @@ void jit_apply_raw(jit_type_t signature, void *func,
 }
 
 /*@
- * @deftypefun int jit_raw_supported (jit_type_t signature)
+ * @deftypefun int jit_raw_supported (jit_type_t @var{signature})
  * Determine if @code{jit_apply_raw} can be used to call functions
  * with a particular signature.  Returns zero if not.
  * @end deftypefun
@@ -559,7 +579,7 @@ static void closure_handler(jit_closure_t closure, void *apply_args)
 	unsigned int num_params;
 	unsigned int param;
 	jit_apply_return apply_return;
-	int is_float_return;
+	_jit_apply_return_type return_type;
 
 	/* Initialize the argument parser */
 	jit_apply_parser_init(&parser, closure->signature, apply_args);
@@ -677,9 +697,12 @@ static void closure_handler(jit_closure_t closure, void *apply_args)
 			case JIT_TYPE_STRUCT:
 			case JIT_TYPE_UNION:
 			{
-				jit_apply_parser_get_struct
-					(&parser, jit_type_get_size(type),
-		 			 jit_type_get_alignment(type), temp_arg);
+#ifdef HAVE_JIT_BUILTIN_APPLY_STRUCT
+				_jit_builtin_apply_get_struct(&parser, temp_arg, type);
+#else
+				jit_apply_parser_get_struct(&parser, jit_type_get_size(type),
+							    jit_type_get_alignment(type), temp_arg);
+#endif
 			}
 			break;
 		}
@@ -698,7 +721,7 @@ static void closure_handler(jit_closure_t closure, void *apply_args)
 	/* Set up the "apply return" buffer */
 	jit_memzero(&apply_return, sizeof(apply_return));
 	type = jit_type_normalize(jit_type_get_return(signature));
-	is_float_return = 0;
+	return_type = _JIT_APPLY_RETURN_TYPE_OTHER;
 	if(type)
 	{
 		switch(type->kind)
@@ -763,7 +786,7 @@ static void closure_handler(jit_closure_t closure, void *apply_args)
 			{
 				jit_apply_return_set_float32
 					(&apply_return, *((jit_float32 *)return_buffer));
-				is_float_return = 1;
+				return_type = _JIT_APPLY_RETURN_TYPE_FLOAT32;
 			}
 			break;
 
@@ -771,7 +794,7 @@ static void closure_handler(jit_closure_t closure, void *apply_args)
 			{
 				jit_apply_return_set_float64
 					(&apply_return, *((jit_float64 *)return_buffer));
-				is_float_return = 1;
+				return_type = _JIT_APPLY_RETURN_TYPE_FLOAT64;
 			}
 			break;
 
@@ -779,7 +802,7 @@ static void closure_handler(jit_closure_t closure, void *apply_args)
 			{
 				jit_apply_return_set_nfloat
 					(&apply_return, *((jit_nfloat *)return_buffer));
-				is_float_return = 1;
+				return_type = _JIT_APPLY_RETURN_TYPE_NFLOAT;
 			}
 			break;
 
@@ -797,23 +820,40 @@ static void closure_handler(jit_closure_t closure, void *apply_args)
 	}
 
 	/* Return the result to the caller */
-	if(!is_float_return)
+	switch(return_type)
 	{
-		jit_builtin_return_int(&apply_return);
-	}
-	else
-	{
-		jit_builtin_return_float(&apply_return);
+		case _JIT_APPLY_RETURN_TYPE_FLOAT32:
+		{
+			jit_builtin_return_float(&apply_return);
+		}
+		break;
+
+		case _JIT_APPLY_RETURN_TYPE_FLOAT64:
+		{
+			jit_builtin_return_double(&apply_return);
+		}
+		break;
+
+		case _JIT_APPLY_RETURN_TYPE_NFLOAT:
+		{
+			jit_builtin_return_nfloat(&apply_return);
+		}
+		break;
+
+		default:
+		{
+			jit_builtin_return_int(&apply_return);
+		}
 	}
 }
 
 #endif /* jit_closure_size */
 
 /*@
- * @deftypefun {void *} jit_closure_create (jit_context_t context, jit_type_t signature, {jit_closure_func} func, {void *} user_data)
+ * @deftypefun {void *} jit_closure_create (jit_context_t @var{context}, jit_type_t @var{signature}, jit_closure_func @var{func}, void *@var{user_data})
  * Create a closure from a function signature, a closure handling function,
  * and a user data value.  Returns NULL if out of memory, or if closures are
- * not supported.  The @code{func} argument should have the following
+ * not supported.  The @var{func} argument should have the following
  * prototype:
  *
  * @example
@@ -826,7 +866,7 @@ static void closure_handler(jit_closure_t closure, void *apply_args)
  * @code{jit_closure_va_list_t} value for accessing the remainder of
  * the arguments.
  *
- * The memory for the closure will be reclaimed when the @code{context}
+ * The memory for the closure will be reclaimed when the @var{context}
  * is destroyed.
  * @end deftypefun
 @*/
@@ -898,14 +938,14 @@ int jit_closures_supported(void)
 }
 
 /*@
- * @deftypefun jit_nint jit_closure_va_get_nint (jit_closure_va_list_t va)
- * @deftypefunx jit_nuint jit_closure_va_get_nuint (jit_closure_va_list_t va)
- * @deftypefunx jit_long jit_closure_va_get_long (jit_closure_va_list_t va)
- * @deftypefunx jit_ulong jit_closure_va_get_ulong (jit_closure_va_list_t va)
- * @deftypefunx jit_float32 jit_closure_va_get_float32 (jit_closure_va_list_t va)
- * @deftypefunx jit_float64 jit_closure_va_get_float64 (jit_closure_va_list_t va)
- * @deftypefunx jit_nfloat jit_closure_va_get_nfloat (jit_closure_va_list_t va)
- * @deftypefunx {void *} jit_closure_va_get_ptr (jit_closure_va_list_t va)
+ * @deftypefun jit_nint jit_closure_va_get_nint (jit_closure_va_list_t @var{va})
+ * @deftypefunx jit_nuint jit_closure_va_get_nuint (jit_closure_va_list_t @var{va})
+ * @deftypefunx jit_long jit_closure_va_get_long (jit_closure_va_list_t @var{va})
+ * @deftypefunx jit_ulong jit_closure_va_get_ulong (jit_closure_va_list_t @var{va})
+ * @deftypefunx jit_float32 jit_closure_va_get_float32 (jit_closure_va_list_t @var{va})
+ * @deftypefunx jit_float64 jit_closure_va_get_float64 (jit_closure_va_list_t @var{va})
+ * @deftypefunx jit_nfloat jit_closure_va_get_nfloat (jit_closure_va_list_t @var{va})
+ * @deftypefunx {void *} jit_closure_va_get_ptr (jit_closure_va_list_t @var{va})
  * Get the next value of a specific type from a closure's variable arguments.
  * @end deftypefun
 @*/
@@ -966,15 +1006,19 @@ void *jit_closure_va_get_ptr(jit_closure_va_list_t va)
 }
 
 /*@
- * @deftypefun void jit_closure_va_get_struct (jit_closure_va_list_t va, void *buf, jit_type_t type)
- * Get a structure or union value of a specific @code{type} from a closure's
- * variable arguments, and copy it into @code{buf}.
+ * @deftypefun void jit_closure_va_get_struct (jit_closure_va_list_t @var{va}, void *@var{buf}, jit_type_t @var{type})
+ * Get a structure or union value of a specific @var{type} from a closure's
+ * variable arguments, and copy it into @var{buf}.
  * @end deftypefun
 @*/
 void jit_closure_va_get_struct
 	(jit_closure_va_list_t va, void *buf, jit_type_t type)
 {
+#ifdef HAVE_JIT_BUILTIN_APPLY_STRUCT
+	_jit_builtin_apply_get_struct(&(va->builder), buf, type);
+#else
 	jit_apply_parser_get_struct
 		(&(va->builder), jit_type_get_size(type),
 		 jit_type_get_alignment(type), buf);
+#endif
 }
