@@ -1,21 +1,23 @@
 /*
  * jit-cache.h - Translated method cache implementation.
  *
- * Copyright (C) 2002, 2004  Southern Storm Software, Pty Ltd.
+ * Copyright (C) 2002, 2004, 2008  Southern Storm Software, Pty Ltd.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This file is part of the libjit library.
  *
- * This program is distributed in the hope that it will be useful,
+ * The libjit library is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation, either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * The libjit library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with the libjit library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #ifndef	_JIT_CACHE_H
@@ -35,7 +37,7 @@ typedef struct jit_cache *jit_cache_t;
  */
 typedef struct
 {
-	jit_cache_t	    cache;			/* Cache this position is attached to */
+	jit_cache_t    cache;			/* Cache this position is attached to */
 	unsigned char  *ptr;			/* Current code pointer */
 	unsigned char  *limit;			/* Limit of the current page */
 
@@ -45,9 +47,14 @@ typedef struct
  * Create a method cache.  Returns NULL if out of memory.
  * If "limit" is non-zero, then it specifies the maximum
  * size of the cache in bytes.  If "cache_page_size" is
- * non-zero, then it indicates the size of each cache page.
+ * non-zero, then it indicates the dafault/minimum cache
+ * page size.  If "max_page_factor" is not zero, then it
+ * indicates the maximum cache page size as multiple of
+ * "max_page_factor" and "cache_page_size".
  */
-jit_cache_t _jit_cache_create(long limit, long cache_page_size);
+jit_cache_t _jit_cache_create(long limit,
+			      long cache_page_size,
+			      int max_page_factor);
 
 /*
  * Destroy a method cache.
@@ -60,22 +67,30 @@ void _jit_cache_destroy(jit_cache_t cache);
  */
 int _jit_cache_is_full(jit_cache_t cache, jit_cache_posn *posn);
 
+/*
+ * Return values for "_jit_cache_start_method" and "_jit_cache_end_method".
+ */
+#define	JIT_CACHE_OK		0		/* Function is OK */
+#define	JIT_CACHE_RESTART	1		/* Restart is required */
+#define	JIT_CACHE_TOO_BIG	2		/* Function is too big for the cache */
+
 /*
  * Start output of a method, returning a cache position.
+ * The "page_factor" value should be equal to zero unless
+ * the method is being recompiled because it did not fit
+ * into the current page. In the later case the value
+ * should gradually increase until either the methods fits
+ * or the maximum page factor value is exceeded.
  * The "align" value indicates the default alignment for
  * the start of the method.  The "cookie" value is a
  * cookie for referring to the method.  Returns the
  * method entry point, or NULL if the cache is full.
  */
-void *_jit_cache_start_method(jit_cache_t cache, jit_cache_posn *posn,
-					      	  int align, void *cookie);
-
-/*
- * Return values for "_jit_cache_end_method".
- */
-#define	JIT_CACHE_END_OK		0		/* Function is OK */
-#define	JIT_CACHE_END_RESTART	1		/* Restart is required */
-#define	JIT_CACHE_END_TOO_BIG	2		/* Function is too big for the cache */
+int _jit_cache_start_method(jit_cache_t cache,
+			    jit_cache_posn *posn,
+			    int page_factor,
+			    int align,
+			    void *cookie);
 
 /*
  * End output of a method.  Returns zero if a restart.
