@@ -3,19 +3,21 @@
  *
  * Copyright (C) 2004  Southern Storm Software, Pty Ltd.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This file is part of the libjit library.
  *
- * This program is distributed in the hope that it will be useful,
+ * The libjit library is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation, either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * The libjit library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with the libjit library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "jit-internal.h"
@@ -88,12 +90,12 @@ jit_context_t jit_context_create(void)
 	jit_mutex_create(&(context->cache_lock));
 	context->functions = 0;
 	context->last_function = 0;
-	context->on_demand_driver = 0;
+	context->on_demand_driver = _jit_function_compile_on_demand;
 	return context;
 }
 
 /*@
- * @deftypefun void jit_context_destroy (jit_context_t context)
+ * @deftypefun void jit_context_destroy (jit_context_t @var{context})
  * Destroy a JIT context block and everything that is associated with it.
  * It is very important that no threads within the program are currently
  * running compiled code when this function is called.
@@ -124,7 +126,7 @@ void jit_context_destroy(jit_context_t context)
 }
 
 /*@
- * @deftypefun int jit_context_supports_threads (jit_context_t context)
+ * @deftypefun int jit_context_supports_threads (jit_context_t @var{context})
  * Determine if the JIT supports threads.
  * @end deftypefun
 @*/
@@ -134,7 +136,7 @@ int jit_context_supports_threads(jit_context_t context)
 }
 
 /*@
- * @deftypefun void jit_context_build_start (jit_context_t context)
+ * @deftypefun void jit_context_build_start (jit_context_t @var{context})
  * This routine should be called before you start building a function
  * to be JIT'ed.  It acquires a lock on the context to prevent other
  * threads from accessing the build process, since only one thread
@@ -147,7 +149,7 @@ void jit_context_build_start(jit_context_t context)
 }
 
 /*@
- * @deftypefun void jit_context_build_end (jit_context_t context)
+ * @deftypefun void jit_context_build_end (jit_context_t @var{context})
  * This routine should be called once you have finished building
  * and compiling a function and are ready to resume normal execution.
  * This routine will release the build lock, allowing other threads
@@ -160,7 +162,7 @@ void jit_context_build_end(jit_context_t context)
 }
 
 /*@
- * @deftypefun void jit_context_set_on_demand_driver (jit_context_t context, jit_on_demand_driver_func driver)
+ * @deftypefun void jit_context_set_on_demand_driver (jit_context_t @var{context}, jit_on_demand_driver_func @var{driver})
  * Specify the C function to be called to drive on-demand compilation.
  *
  * When on-demand compilation is requested the default driver provided by
@@ -204,11 +206,18 @@ void jit_context_build_end(jit_context_t context)
 @*/
 void jit_context_set_on_demand_driver(jit_context_t context, jit_on_demand_driver_func driver)
 {
-	context->on_demand_driver = driver;
+	if (driver)
+	{
+		context->on_demand_driver = driver;
+	}
+	else
+	{
+		context->on_demand_driver = _jit_function_compile_on_demand;
+	}
 }
 
 /*@
- * @deftypefun int jit_context_set_meta (jit_context_t context, int type, {void *} data, jit_meta_free_func free_data)
+ * @deftypefun int jit_context_set_meta (jit_context_t @var{context}, int @var{type}, void *@var{data}, jit_meta_free_func @var{free_data})
  * Tag a context with some metadata.  Returns zero if out of memory.
  *
  * Metadata may be used to store dependency graphs, branch prediction
@@ -217,7 +226,7 @@ void jit_context_set_on_demand_driver(jit_context_t context, jit_on_demand_drive
  * to store information about the context that is specific to the
  * virtual machine or language.
  *
- * If the @code{type} already has some metadata associated with it, then
+ * If the @var{type} already has some metadata associated with it, then
  * the previous value will be freed.
  * @end deftypefun
 @*/
@@ -229,7 +238,7 @@ int jit_context_set_meta
 }
 
 /*@
- * @deftypefun int jit_context_set_meta_numeric (jit_context_t context, int type, jit_nuint data)
+ * @deftypefun int jit_context_set_meta_numeric (jit_context_t @var{context}, int @var{type}, jit_nuint @var{data})
  * Tag a context with numeric metadata.  Returns zero if out of memory.
  * This function is more convenient for accessing the context's
  * special option values:
@@ -280,9 +289,9 @@ int jit_context_set_meta_numeric
 }
 
 /*@
- * @deftypefun {void *} jit_context_get_meta (jit_context_t context, int type)
+ * @deftypefun {void *} jit_context_get_meta (jit_context_t @var{context}, int @var{type})
  * Get the metadata associated with a particular tag.  Returns NULL
- * if @code{type} does not have any metadata associated with it.
+ * if @var{type} does not have any metadata associated with it.
  * @end deftypefun
 @*/
 void *jit_context_get_meta(jit_context_t context, int type)
@@ -291,9 +300,9 @@ void *jit_context_get_meta(jit_context_t context, int type)
 }
 
 /*@
- * @deftypefun jit_nuint jit_context_get_meta_numeric (jit_context_t context, int type)
+ * @deftypefun jit_nuint jit_context_get_meta_numeric (jit_context_t @var{context}, int @var{type})
  * Get the metadata associated with a particular tag.  Returns zero
- * if @code{type} does not have any metadata associated with it.
+ * if @var{type} does not have any metadata associated with it.
  * This version is more convenient for the pre-defined numeric option values.
  * @end deftypefun
 @*/
@@ -303,9 +312,9 @@ jit_nuint jit_context_get_meta_numeric(jit_context_t context, int type)
 }
 
 /*@
- * @deftypefun void jit_context_free_meta (jit_context_t context, int type)
+ * @deftypefun void jit_context_free_meta (jit_context_t @var{context}, int @var{type})
  * Free metadata of a specific type on a context.  Does nothing if
- * the @code{type} does not have any metadata associated with it.
+ * the @var{type} does not have any metadata associated with it.
  * @end deftypefun
 @*/
 void jit_context_free_meta(jit_context_t context, int type)
@@ -321,7 +330,9 @@ struct jit_cache *_jit_context_get_cache(jit_context_t context)
 			((long)jit_context_get_meta_numeric
 				(context, JIT_OPTION_CACHE_LIMIT),
 			 (long)jit_context_get_meta_numeric
-				(context, JIT_OPTION_CACHE_PAGE_SIZE));
+			 	(context, JIT_OPTION_CACHE_PAGE_SIZE),
+			 (int)jit_context_get_meta_numeric
+				(context, JIT_OPTION_CACHE_MAX_PAGE_FACTOR));
 	}
 	return context->cache;
 }
