@@ -1,21 +1,23 @@
 /*
  * jit-function.c - Functions for manipulating function blocks.
  *
- * Copyright (C) 2004  Southern Storm Software, Pty Ltd.
+ * Copyright (C) 2004, 2006-2008  Southern Storm Software, Pty Ltd.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This file is part of the libjit library.
  *
- * This program is distributed in the hope that it will be useful,
+ * The libjit library is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation, either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * The libjit library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with the libjit library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "jit-internal.h"
@@ -29,7 +31,7 @@
 #endif
 
 /*@
- * @deftypefun jit_function_t jit_function_create (jit_context_t context, jit_type_t signature)
+ * @deftypefun jit_function_t jit_function_create (jit_context_t @var{context}, jit_type_t @var{signature})
  * Create a new function block and associate it with a JIT context.
  * Returns NULL if out of memory.
  *
@@ -49,10 +51,7 @@
 jit_function_t jit_function_create(jit_context_t context, jit_type_t signature)
 {
 	jit_function_t func;
-#if defined(jit_redirector_size)
-	jit_on_demand_driver_func on_demand_driver;
-#endif
-#if defined(jit_redirector_size) || defined(jit_indirector_size)
+#if !defined(JIT_BACKEND_INTERP) && (defined(jit_redirector_size) || defined(jit_indirector_size))
 	jit_cache_t cache;
 #endif
 
@@ -63,7 +62,7 @@ jit_function_t jit_function_create(jit_context_t context, jit_type_t signature)
 		return 0;
 	}
 
-#if defined(jit_redirector_size) || defined(jit_indirector_size)
+#if !defined(JIT_BACKEND_INTERP) && (defined(jit_redirector_size) || defined(jit_indirector_size))
 	/* TODO: if the function is destroyed the redirector and indirector memory
 	   is leaked */
 
@@ -101,27 +100,22 @@ jit_function_t jit_function_create(jit_context_t context, jit_type_t signature)
 # endif
 
 	jit_mutex_unlock(&(context->cache_lock));
-#endif
+#endif /* !defined(JIT_BACKEND_INTERP) && (defined(jit_redirector_size) || defined(jit_indirector_size)) */
 
 	/* Initialize the function block */
 	func->context = context;
 	func->signature = jit_type_copy(signature);
 
-#if defined(jit_redirector_size)
+#if !defined(JIT_BACKEND_INTERP) && defined(jit_redirector_size)
 	/* If we aren't using interpretation, then point the function's
 	   initial entry point at the redirector, which in turn will
 	   invoke the on-demand compiler */
-	on_demand_driver = context->on_demand_driver;
-	if(!on_demand_driver)
-	{
-		on_demand_driver = _jit_function_compile_on_demand;
-	}
 	func->entry_point = _jit_create_redirector
-		(func->redirector, (void *) on_demand_driver,
+		(func->redirector, (void *) context->on_demand_driver,
 		 func, jit_type_get_abi(signature));
 	jit_flush_exec(func->redirector, jit_redirector_size);
 #endif
-# if defined(jit_indirector_size)
+# if !defined(JIT_BACKEND_INTERP) && defined(jit_indirector_size)
 	_jit_create_indirector(func->indirector, (void**) &(func->entry_point));
 	jit_flush_exec(func->indirector, jit_indirector_size);
 # endif
@@ -144,10 +138,10 @@ jit_function_t jit_function_create(jit_context_t context, jit_type_t signature)
 }
 
 /*@
- * @deftypefun jit_function_t jit_function_create_nested (jit_context_t context, jit_type_t signature, jit_function_t parent)
+ * @deftypefun jit_function_t jit_function_create_nested (jit_context_t @var{context}, jit_type_t @var{signature}, jit_function_t @var{parent})
  * Create a new function block and associate it with a JIT context.
  * In addition, this function is nested inside the specified
- * @code{parent} function and is able to access its parent's
+ * @var{parent} function and is able to access its parent's
  * (and grandparent's) local variables.
  *
  * The front end is responsible for ensuring that the nested function can
@@ -261,14 +255,15 @@ void _jit_function_destroy(jit_function_t func)
 	}
 	_jit_function_free_builder(func);
 	jit_meta_destroy(&(func->meta));
+	jit_type_free(func->signature);
 	jit_free(func);
 }
 
 /*@
- * @deftypefun void jit_function_abandon (jit_function_t func)
+ * @deftypefun void jit_function_abandon (jit_function_t @var{func})
  * Abandon this function during the build process.  This should be called
  * when you detect a fatal error that prevents the function from being
- * properly built.  The @code{func} object is completely destroyed and
+ * properly built.  The @var{func} object is completely destroyed and
  * detached from its owning context.  The function is left alone if
  * it was already compiled.
  * @end deftypefun
@@ -293,7 +288,7 @@ void jit_function_abandon(jit_function_t func)
 }
 
 /*@
- * @deftypefun jit_context_t jit_function_get_context (jit_function_t func)
+ * @deftypefun jit_context_t jit_function_get_context (jit_function_t @var{func})
  * Get the context associated with a function.
  * @end deftypefun
 @*/
@@ -310,7 +305,7 @@ jit_context_t jit_function_get_context(jit_function_t func)
 }
 
 /*@
- * @deftypefun jit_type_t jit_function_get_signature (jit_function_t func)
+ * @deftypefun jit_type_t jit_function_get_signature (jit_function_t @var{func})
  * Get the signature associated with a function.
  * @end deftypefun
 @*/
@@ -327,7 +322,7 @@ jit_type_t jit_function_get_signature(jit_function_t func)
 }
 
 /*@
- * @deftypefun int jit_function_set_meta (jit_function_t func, int type, {void *} data, jit_meta_free_func free_data, int build_only)
+ * @deftypefun int jit_function_set_meta (jit_function_t @var{func}, int @var{type}, void *@var{data}, jit_meta_free_func @var{free_data}, int @var{build_only})
  * Tag a function with some metadata.  Returns zero if out of memory.
  *
  * Metadata may be used to store dependency graphs, branch prediction
@@ -336,13 +331,13 @@ jit_type_t jit_function_get_signature(jit_function_t func)
  * to store information about the function that is specific to the
  * virtual machine or language.
  *
- * If the @code{type} already has some metadata associated with it, then
+ * If the @var{type} already has some metadata associated with it, then
  * the previous value will be freed.
  *
- * If @code{build_only} is non-zero, then the metadata will be freed
+ * If @var{build_only} is non-zero, then the metadata will be freed
  * when the function is compiled with @code{jit_function_compile}.
  * Otherwise the metadata will persist until the JIT context is destroyed,
- * or @code{jit_function_free_meta} is called for the specified @code{type}.
+ * or @code{jit_function_free_meta} is called for the specified @var{type}.
  *
  * Metadata type values of 10000 or greater are reserved for internal use.
  * @end deftypefun
@@ -366,9 +361,9 @@ int jit_function_set_meta(jit_function_t func, int type, void *data,
 }
 
 /*@
- * @deftypefun {void *} jit_function_get_meta (jit_function_t func, int type)
+ * @deftypefun {void *} jit_function_get_meta (jit_function_t @var{func}, int @var{type})
  * Get the metadata associated with a particular tag.  Returns NULL
- * if @code{type} does not have any metadata associated with it.
+ * if @var{type} does not have any metadata associated with it.
  * @end deftypefun
 @*/
 void *jit_function_get_meta(jit_function_t func, int type)
@@ -382,9 +377,9 @@ void *jit_function_get_meta(jit_function_t func, int type)
 }
 
 /*@
- * @deftypefun void jit_function_free_meta (jit_function_t func, int type)
+ * @deftypefun void jit_function_free_meta (jit_function_t @var{func}, int @var{type})
  * Free metadata of a specific type on a function.  Does nothing if
- * the @code{type} does not have any metadata associated with it.
+ * the @var{type} does not have any metadata associated with it.
  * @end deftypefun
 @*/
 void jit_function_free_meta(jit_function_t func, int type)
@@ -397,8 +392,8 @@ void jit_function_free_meta(jit_function_t func, int type)
 }
 
 /*@
- * @deftypefun jit_function_t jit_function_next (jit_context_t context, jit_function_t prev)
- * Iterate over the defined functions in creation order.  The @code{prev}
+ * @deftypefun jit_function_t jit_function_next (jit_context_t @var{context}, jit_function_t @var{prev})
+ * Iterate over the defined functions in creation order.  The @var{prev}
  * argument should be NULL on the first call.  Returns NULL at the end.
  * @end deftypefun
 @*/
@@ -419,12 +414,11 @@ jit_function_t jit_function_next(jit_context_t context, jit_function_t prev)
 }
 
 /*@
- * @deftypefun jit_function_t jit_function_previous (jit_context_t context, jit_function_t prev)
+ * @deftypefun jit_function_t jit_function_previous (jit_context_t @var{context}, jit_function_t @var{prev})
  * Iterate over the defined functions in reverse creation order.
  * @end deftypefun
 @*/
-jit_function_t jit_function_previous(jit_context_t context,
-									 jit_function_t prev)
+jit_function_t jit_function_previous(jit_context_t context, jit_function_t prev)
 {
 	if(prev)
 	{
@@ -441,7 +435,7 @@ jit_function_t jit_function_previous(jit_context_t context,
 }
 
 /*@
- * @deftypefun jit_block_t jit_function_get_entry (jit_function_t func)
+ * @deftypefun jit_block_t jit_function_get_entry (jit_function_t @var{func})
  * Get the entry block for a function.  This is always the first block
  * created by @code{jit_function_create}.
  * @end deftypefun
@@ -459,7 +453,7 @@ jit_block_t jit_function_get_entry(jit_function_t func)
 }
 
 /*@
- * @deftypefun jit_block_t jit_function_get_current (jit_function_t func)
+ * @deftypefun jit_block_t jit_function_get_current (jit_function_t @var{func})
  * Get the current block for a function.  New blocks are created by
  * certain @code{jit_insn_xxx} calls.
  * @end deftypefun
@@ -477,8 +471,8 @@ jit_block_t jit_function_get_current(jit_function_t func)
 }
 
 /*@
- * @deftypefun jit_function_t jit_function_get_nested_parent (jit_function_t func)
- * Get the nested parent for a function, or NULL if @code{func}
+ * @deftypefun jit_function_t jit_function_get_nested_parent (jit_function_t @var{func})
+ * Get the nested parent for a function, or NULL if @var{func}
  * does not have a nested parent.
  * @end deftypefun
 @*/
@@ -518,16 +512,6 @@ static void compile_block(jit_gencode_t gen, jit_function_t func,
 		printf("Start of binary code: 0x%08x\n", p1);
 #endif
 
-		/* The cache is full. */
-		if(!jit_cache_check_for_n(&gen->posn, 1))
-		{
-#ifdef _JIT_COMPILE_DEBUG
-			printf("No space left in the code cache.\n\n");
-			fflush(stdout);
-#endif
-			return;
-		}
-
 		switch(insn->opcode)
 		{
 			case JIT_OP_NOP:		break;		/* Ignore NOP's */
@@ -549,6 +533,7 @@ static void compile_block(jit_gencode_t gen, jit_function_t func,
 				_jit_regs_set_incoming
 					(gen, (int)jit_value_get_nint_constant(insn->value2),
 					 insn->value1);
+				_jit_gen_insn(gen, func, block, insn);
 			}
 			break;
 #endif
@@ -656,10 +641,6 @@ cleanup_on_restart(jit_gencode_t gen, jit_function_t func)
 	jit_insn_iter_t iter;
 	jit_insn_t insn;
 
-#ifdef _JIT_COMPILE_DEBUG
-	printf("\n*** Restart compilation ***\n\n");
-#endif
-
 	block = 0;
 	while((block = jit_block_next(func, block)) != 0)
 	{
@@ -711,6 +692,9 @@ cleanup_on_restart(jit_gencode_t gen, jit_function_t func)
 	{
 		gen->touched = gen->permanent;
 	}
+
+	/* Reset the epilog fixup list */
+	gen->epilog_fixup = 0;
 }
 
 /*
@@ -721,27 +705,27 @@ compile(jit_function_t func, void **entry_point)
 {
 	struct jit_gencode gen;
 	jit_cache_t cache;
-	void *start;
-	void *end;
+	unsigned char *start;
+	unsigned char *end;
 	jit_block_t block;
+	int page_factor;
 	int result;
-#ifdef JIT_PROLOG_SIZE
-	int have_prolog;
-#endif
-
-	/* We need the cache lock while we are compiling the function */
-	jit_mutex_lock(&(func->context->cache_lock));
-
-	/* Get the method cache */
-	cache = _jit_context_get_cache(func->context);
-	if(!cache)
-	{
-		jit_mutex_unlock(&(func->context->cache_lock));
-		return 0;
-	}
 
 	/* Initialize the code generation state */
 	jit_memzero(&gen, sizeof(gen));
+	page_factor = 0;
+	start = 0;
+	end = 0;
+
+	/* Intuit "nothrow" and "noreturn" flags for this function */
+	if(!(func->builder->may_throw))
+	{
+		func->no_throw = 1;
+	}
+	if(!(func->builder->ordinary_return))
+	{
+		func->no_return = 1;
+	}
 
 	/* Compute liveness and "next use" information for this function */
 	_jit_function_compute_liveness(func);
@@ -751,28 +735,44 @@ compile(jit_function_t func, void **entry_point)
 	_jit_regs_alloc_global(&gen, func);
 #endif
 
-	/* We may need to perform output twice, if the first attempt fails
-	   due to a lack of space in the current method cache page */
-	do
-	{
+	/* We need the cache lock while we are compiling the function */
+	jit_mutex_lock(&(func->context->cache_lock));
+
 #ifdef _JIT_COMPILE_DEBUG
-		printf("\n*** Start compilation ***\n\n");
-		func->builder->block_count = 0;
-		func->builder->insn_count = 0;
+	printf("\n*** Start compilation ***\n\n");
+	func->builder->block_count = 0;
+	func->builder->insn_count = 0;
 #endif
 
-		/* Start function output to the cache */
-		start = _jit_cache_start_method
-			(cache, &(gen.posn), JIT_FUNCTION_ALIGNMENT, func);
-		if(!start)
-		{
-#ifdef jit_extra_gen_cleanup
-			/* Clean up the extra code generation state */
-			jit_extra_gen_cleanup(gen);
-#endif
-			jit_mutex_unlock(&(func->context->cache_lock));
-			return 0;
-		}
+	/* Get the method cache */
+	cache = _jit_context_get_cache(func->context);
+	if(!cache)
+	{
+		jit_mutex_unlock(&(func->context->cache_lock));
+		return 0;
+	}
+
+	/* Start function output to the cache */
+	result = _jit_cache_start_method(cache, &(gen.posn),
+					 page_factor++,
+					 JIT_FUNCTION_ALIGNMENT, func);
+	if (result == JIT_CACHE_RESTART)
+	{
+		/* No space left on the current cache page.  Allocate a new one. */
+		result = _jit_cache_start_method(cache, &(gen.posn),
+						 page_factor++,
+						 JIT_FUNCTION_ALIGNMENT, func);
+	}
+	if (result != JIT_CACHE_OK)
+	{
+		/* Failed to allocate any cache space */
+		jit_mutex_unlock(&(func->context->cache_lock));
+		return 0;
+	}
+
+	for(;;)
+	{
+		start = gen.posn.ptr;
 
 #ifdef jit_extra_gen_init
 		/* Initialize information that may need to be reset each loop */
@@ -781,20 +781,13 @@ compile(jit_function_t func, void **entry_point)
 
 #ifdef JIT_PROLOG_SIZE
 		/* Output space for the function prolog */
-		if(jit_cache_check_for_n(&(gen.posn), JIT_PROLOG_SIZE))
+		if(!jit_cache_check_for_n(&(gen.posn), JIT_PROLOG_SIZE))
 		{
-			gen.posn.ptr += JIT_PROLOG_SIZE;
-			have_prolog = 1;
+			/* No space left on the current cache page.  Restart. */
+			jit_cache_mark_full(&(gen.posn));
+			goto restart;
 		}
-		else
-		{
-			have_prolog = 0;
-		}
-#endif
-
-		/* Clear the register assignments for the first block */
-#ifndef JIT_BACKEND_INTERP
-		_jit_regs_init_for_block(&gen);
+		gen.posn.ptr += JIT_PROLOG_SIZE;
 #endif
 
 		/* Generate code for the blocks in the function */
@@ -809,22 +802,29 @@ compile(jit_function_t func, void **entry_point)
 
 			/* Notify the back end that the block is starting */
 			_jit_gen_start_block(&gen, block);
-	
+
+#ifndef JIT_BACKEND_INTERP
+			/* Clear the local register assignments */
+			_jit_regs_init_for_block(&gen);
+#endif
+
 			/* Generate the block's code */
 			compile_block(&gen, func, block);
 
-			/* Spill all live register values back to their frame positions */
 #ifndef JIT_BACKEND_INTERP
+			/* Spill all live register values back to their frame positions */
 			_jit_regs_spill_all(&gen);
 #endif
 
 			/* Notify the back end that the block is finished */
 			_jit_gen_end_block(&gen, block);
-	
-			/* Clear the local register assignments, ready for the next block */
-#ifndef JIT_BACKEND_INTERP
-			_jit_regs_init_for_block(&gen);
-#endif
+
+			/* Stop code generation if the cache page is full */
+			if(_jit_cache_is_full(cache, &(gen.posn)))
+			{
+				/* No space left on the current cache page.  Restart. */
+				goto restart;
+			}
 		}
 
 		/* Output the function epilog.  All return paths will jump to here */
@@ -833,13 +833,10 @@ compile(jit_function_t func, void **entry_point)
 
 #ifdef JIT_PROLOG_SIZE
 		/* Back-patch the function prolog and get the real entry point */
-		if(have_prolog)
-		{
-			start = _jit_gen_prolog(&gen, func, start);
-		}
+		start = _jit_gen_prolog(&gen, func, start);
 #endif
 
-#if !defined(jit_redirector_size) || !defined(jit_indirector_size)
+#if !defined(JIT_BACKEND_INTERP) && (!defined(jit_redirector_size) || !defined(jit_indirector_size))
 		/* If the function is recompilable, then we need an extra entry
 		   point to properly redirect previous references to the function */
 		if(func->is_recompilable && !func->indirector)
@@ -850,17 +847,38 @@ compile(jit_function_t func, void **entry_point)
 		}
 #endif
 
+	restart:
 		/* End the function's output process */
 		result = _jit_cache_end_method(&(gen.posn));
-
-		/* If we need to restart on a different cache page, then clean up
-		   the compilation state  */
-		if(result == JIT_CACHE_END_RESTART)
+		if(result != JIT_CACHE_RESTART)
 		{
-			cleanup_on_restart(&gen, func);
+			break;
 		}
+
+		/* Clean up the compilation state before restart */
+		cleanup_on_restart(&gen, func);
+
+#ifdef _JIT_COMPILE_DEBUG
+		printf("\n*** Restart compilation ***\n\n");
+		func->builder->block_count = 0;
+		func->builder->insn_count = 0;
+#endif
+
+		/* Restart function output to the cache */
+		result = _jit_cache_start_method(cache, &(gen.posn),
+						 page_factor,
+						 JIT_FUNCTION_ALIGNMENT, func);
+		if(result != JIT_CACHE_OK)
+		{
+#ifdef jit_extra_gen_cleanup
+			/* Clean up the extra code generation state */
+			jit_extra_gen_cleanup(gen);
+#endif
+			jit_mutex_unlock(&(func->context->cache_lock));
+			return 0;
+		}
+		page_factor *= 2;
 	}
-	while(result == JIT_CACHE_END_RESTART);
 
 #ifdef jit_extra_gen_cleanup
 	/* Clean up the extra code generation state */
@@ -868,7 +886,7 @@ compile(jit_function_t func, void **entry_point)
 #endif
 
 	/* Bail out if we ran out of memory while translating the function */
-	if(result != JIT_CACHE_END_OK)
+	if(result != JIT_CACHE_OK)
 	{
 		jit_mutex_unlock(&(func->context->cache_lock));
 		return 0;
@@ -876,26 +894,15 @@ compile(jit_function_t func, void **entry_point)
 
 #ifndef JIT_BACKEND_INTERP
 	/* Perform a CPU cache flush, to make the code executable */
-	jit_flush_exec(start, (unsigned int)(((unsigned char *)end) -
-										 ((unsigned char *)start)));
+	jit_flush_exec(start, (unsigned int)(end - start));
 #endif
 
-	/* Intuit "nothrow" and "noreturn" flags for this function */
-	if(!(func->builder->may_throw))
-	{
-		func->no_throw = 1;
-	}
-	if(!(func->builder->ordinary_return))
-	{
-		func->no_return = 1;
-	}
-
-	/* Free the builder structure, which we no longer require */
-	_jit_function_free_builder(func);
-
 	/* The function has been compiled successfully */
 	jit_mutex_unlock(&(func->context->cache_lock));
 
+	/* Free the builder structure, which we no longer require */
+	_jit_function_free_builder(func);
+
 	/* Record the entry point */
 	if(entry_point)
 	{
@@ -917,7 +924,7 @@ struct jit_cache_eh
 };
 
 /*@
- * @deftypefun int jit_function_compile (jit_function_t func)
+ * @deftypefun int jit_function_compile (jit_function_t @var{func})
  * Compile a function to its executable form.  If the function was
  * already compiled, then do nothing.  Returns zero on error.
  *
@@ -966,7 +973,7 @@ int jit_function_compile(jit_function_t func)
 }
 
 /*@
- * @deftypefun int jit_function_compile_entry (jit_function_t func, void **entry_point)
+ * @deftypefun int jit_function_compile_entry (jit_function_t @var{func}, void **@var{entry_point})
  * Compile a function to its executable form but do not make it
  * available for invocation yet. It may be made available later
  * with @code{jit_function_setup_entry}.
@@ -1007,10 +1014,10 @@ jit_function_compile_entry(jit_function_t func, void **entry_point)
 }
 
 /*@
- * @deftypefun int jit_function_setup_entry (jit_function_t func, void *entry_point)
+ * @deftypefun int jit_function_setup_entry (jit_function_t @var{func}, void *@var{entry_point})
  * Make a function compiled with @code{jit_function_compile_entry}
  * available for invocation and free the resources used for
- * compilation. If @code{entry_point} is null then it only
+ * compilation. If @var{entry_point} is null then it only
  * frees the resources.
  * @end deftypefun
 @*/
@@ -1032,59 +1039,7 @@ jit_function_setup_entry(jit_function_t func, void *entry_point)
 }
 
 /*@
- * @deftypefun int jit_function_recompile (jit_function_t func)
- * Force @code{func} to be recompiled, by calling its on-demand
- * compiler again.  It is highly recommended that you set the
- * recompilable flag with @code{jit_function_set_recompilable}
- * when you initially create the function.
- *
- * This function returns one of @code{JIT_RESULT_OK},
- * @code{JIT_RESULT_COMPILE_ERROR}, or @code{JIT_RESULT_OUT_OF_MEMORY}.
- * @end deftypefun
-@*/
-int jit_function_recompile(jit_function_t func)
-{
-	int result;
-
-	/* Lock down the context */
-	jit_context_build_start(func->context);
-
-	/* Call the user's on-demand compiler if we don't have a builder yet.
-	   Bail out with an error if there is no on-demand compiler */
-	if(!(func->builder))
-	{
-		if(func->on_demand)
-		{
-			result = (*(func->on_demand))(func);
-			if(result != JIT_RESULT_OK)
-			{
-				_jit_function_free_builder(func);
-				jit_context_build_end(func->context);
-				return result;
-			}
-		}
-		else
-		{
-			jit_context_build_end(func->context);
-			return JIT_RESULT_COMPILE_ERROR;
-		}
-	}
-
-	/* Compile the function */
-	if(!jit_function_compile(func))
-	{
-		_jit_function_free_builder(func);
-		jit_context_build_end(func->context);
-		return JIT_RESULT_OUT_OF_MEMORY;
-	}
-
-	/* Unlock the context and report that we are ready to go */
-	jit_context_build_end(func->context);
-	return JIT_RESULT_OK;
-}
-
-/*@
- * @deftypefun int jit_function_is_compiled (jit_function_t func)
+ * @deftypefun int jit_function_is_compiled (jit_function_t @var{func})
  * Determine if a function has already been compiled.
  * @end deftypefun
 @*/
@@ -1101,7 +1056,7 @@ int jit_function_is_compiled(jit_function_t func)
 }
 
 /*@
- * @deftypefun int jit_function_set_recompilable (jit_function_t func)
+ * @deftypefun int jit_function_set_recompilable (jit_function_t @var{func})
  * Mark this function as a candidate for recompilation.  That is,
  * it is possible that we may call @code{jit_function_compile}
  * more than once, to re-optimize an existing function.
@@ -1123,7 +1078,7 @@ void jit_function_set_recompilable(jit_function_t func)
 }
 
 /*@
- * @deftypefun void jit_function_clear_recompilable (jit_function_t func)
+ * @deftypefun void jit_function_clear_recompilable (jit_function_t @var{func})
  * Clear the recompilable flag on this function.  Normally you would use
  * this once you have decided that the function has been optimized enough,
  * and that you no longer intend to call @code{jit_function_compile} again.
@@ -1144,7 +1099,7 @@ void jit_function_clear_recompilable(jit_function_t func)
 }
 
 /*@
- * @deftypefun int jit_function_is_recompilable (jit_function_t func)
+ * @deftypefun int jit_function_is_recompilable (jit_function_t @var{func})
  * Determine if this function is recompilable.
  * @end deftypefun
 @*/
@@ -1179,7 +1134,7 @@ static void function_closure(jit_type_t signature, void *result,
 #endif /* JIT_BACKEND_INTERP */
 
 /*@
- * @deftypefun {void *} jit_function_to_closure (jit_function_t func)
+ * @deftypefun {void *} jit_function_to_closure (jit_function_t @var{func})
  * Convert a compiled function into a closure that can called directly
  * from C.  Returns NULL if out of memory, or if closures are not
  * supported on this platform.
@@ -1212,7 +1167,7 @@ void *jit_function_to_closure(jit_function_t func)
 }
 
 /*@
- * @deftypefun jit_function_t jit_function_from_closure (jit_context_t context, {void *} closure)
+ * @deftypefun jit_function_t jit_function_from_closure (jit_context_t @var{context}, void *@var{closure})
  * Convert a closure back into a function.  Returns NULL if the
  * closure does not correspond to a function in the specified context.
  * @end deftypefun
@@ -1229,11 +1184,11 @@ jit_function_t jit_function_from_closure(jit_context_t context, void *closure)
 }
 
 /*@
- * @deftypefun jit_function_t jit_function_from_pc (jit_context_t context, {void *} pc, {void **} handler)
+ * @deftypefun jit_function_t jit_function_from_pc (jit_context_t @var{context}, void *@var{pc}, void **@var{handler})
  * Get the function that contains the specified program counter location.
  * Also return the address of the @code{catch} handler for the same location.
  * Returns NULL if the program counter does not correspond to a function
- * under the control of @code{context}.
+ * under the control of @var{context}.
  * @end deftypefun
 @*/
 jit_function_t jit_function_from_pc
@@ -1275,7 +1230,7 @@ jit_function_t jit_function_from_pc
 }
 
 /*@
- * @deftypefun {void *} jit_function_to_vtable_pointer (jit_function_t func)
+ * @deftypefun {void *} jit_function_to_vtable_pointer (jit_function_t @var{func})
  * Return a pointer that is suitable for referring to this function
  * from a vtable.  Such pointers should only be used with the
  * @code{jit_insn_call_vtable} instruction.
@@ -1307,7 +1262,7 @@ void *jit_function_to_vtable_pointer(jit_function_t func)
 }
 
 /*@
- * @deftypefun jit_function_t jit_function_from_vtable_pointer (jit_context_t context, {void *} vtable_pointer)
+ * @deftypefun jit_function_t jit_function_from_vtable_pointer (jit_context_t @var{context}, void *@var{vtable_pointer})
  * Convert a vtable_pointer back into a function.  Returns NULL if the
  * vtable_pointer does not correspond to a function in the specified context.
  * @end deftypefun
@@ -1335,8 +1290,8 @@ jit_function_t jit_function_from_vtable_pointer(jit_context_t context, void *vta
 }
 
 /*@
- * @deftypefun void jit_function_set_on_demand_compiler (jit_function_t func, jit_on_demand_func on_demand)
- * Specify the C function to be called when @code{func} needs to be
+ * @deftypefun void jit_function_set_on_demand_compiler (jit_function_t @var{func}, jit_on_demand_func @var{on_demand})
+ * Specify the C function to be called when @var{func} needs to be
  * compiled on-demand.  This should be set just after the function
  * is created, before any build or compile processes begin.
  *
@@ -1390,7 +1345,7 @@ jit_function_set_on_demand_compiler(jit_function_t func, jit_on_demand_func on_d
 }
 
 /*@
- * @deftypefun jit_on_demand_func jit_function_get_on_demand_compiler (jit_function_t func)
+ * @deftypefun jit_on_demand_func jit_function_get_on_demand_compiler (jit_function_t @var{func})
  * Returns function's on-demand compiler.
  * @end deftypefun
 @*/
@@ -1461,9 +1416,9 @@ void *_jit_function_compile_on_demand(jit_function_t func)
 }
 
 /*@
- * @deftypefun int jit_function_apply (jit_function_t func, {void **} args, {void *} return_area)
- * Call the function @code{func} with the supplied arguments.  Each element
- * in @code{args} is a pointer to one of the arguments, and @code{return_area}
+ * @deftypefun int jit_function_apply (jit_function_t @var{func}, void **@var{args}, void *@var{return_area})
+ * Call the function @var{func} with the supplied arguments.  Each element
+ * in @var{args} is a pointer to one of the arguments, and @var{return_area}
  * points to a buffer to receive the return value.  Returns zero if an
  * exception occurred.
  *
@@ -1473,15 +1428,15 @@ void *_jit_function_compile_on_demand(jit_function_t func)
  * is guaranteed to be supported everywhere.
  *
  * Function applications acts as an exception blocker.  If any exceptions
- * occur during the execution of @code{func}, they won't travel up the
+ * occur during the execution of @var{func}, they won't travel up the
  * stack any further than this point.  This prevents ordinary C code
  * from being accidentally presented with a situation that it cannot handle.
  * This blocking protection is not present when a function is invoked
  * via its closure.
  * @end deftypefun
  *
- * @deftypefun int jit_function_apply_vararg (jit_function_t func, jit_type_t signature, {void **} args, {void *} return_area)
- * Call the function @code{func} with the supplied arguments.  There may
+ * @deftypefun int jit_function_apply_vararg (jit_function_t @var{func}, jit_type_t @var{signature}, void **@var{args}, void *@var{return_area})
+ * Call the function @var{func} with the supplied arguments.  There may
  * be more arguments than are specified in the function's original signature,
  * in which case the additional values are passed as variable arguments.
  * This function is otherwise identical to @code{jit_function_apply}.
@@ -1507,9 +1462,6 @@ int jit_function_apply_vararg
 	(jit_function_t func, jit_type_t signature, void **args, void *return_area)
 {
 	struct jit_backtrace call_trace;
-#if defined(jit_redirector_size)
-	jit_on_demand_driver_func on_demand_driver;
-#endif
 	void *entry;
 	jit_jmp_buf jbuf;
 
@@ -1544,16 +1496,7 @@ int jit_function_apply_vararg
 	}
 	else
 	{
-#if defined(jit_redirector_size)
-		on_demand_driver = func->context->on_demand_driver;
-		if(!on_demand_driver)
-		{
-			on_demand_driver = _jit_function_compile_on_demand;
-		}
-		entry = (*on_demand_driver)(func);
-#else
-		entry = _jit_function_compile_on_demand(func);
-#endif
+		entry = (*func->context->on_demand_driver)(func);
 	}
 
 	/* Get the default signature if necessary */
@@ -1577,11 +1520,11 @@ int jit_function_apply_vararg
 #endif /* !JIT_BACKEND_INTERP */
 
 /*@
- * @deftypefun void jit_function_set_optimization_level (jit_function_t func, {unsigned int} level)
- * Set the optimization level for @code{func}.  Increasing values indicate
+ * @deftypefun void jit_function_set_optimization_level (jit_function_t @var{func}, unsigned int @var{level})
+ * Set the optimization level for @var{func}.  Increasing values indicate
  * that the @code{libjit} dynamic compiler should expend more effort to
  * generate better code for this function.  Usually you would increase
- * this value just before forcing @code{func} to recompile.
+ * this value just before forcing @var{func} to recompile.
  *
  * When the optimization level reaches the value returned by
  * @code{jit_function_get_max_optimization_level()}, there is usually
@@ -1590,7 +1533,7 @@ int jit_function_apply_vararg
  *
  * The front end is usually responsible for choosing candidates for
  * function inlining.  If it has identified more such candidates, then
- * it may still want to recompile @code{func} again even once it has
+ * it may still want to recompile @var{func} again even once it has
  * reached the maximum optimization level.
  * @end deftypefun
 @*/
@@ -1609,8 +1552,8 @@ void jit_function_set_optimization_level
 }
 
 /*@
- * @deftypefun {unsigned int} jit_function_get_optimization_level (jit_function_t func)
- * Get the current optimization level for @code{func}.
+ * @deftypefun {unsigned int} jit_function_get_optimization_level (jit_function_t @var{func})
+ * Get the current optimization level for @var{func}.
  * @end deftypefun
 @*/
 unsigned int jit_function_get_optimization_level(jit_function_t func)
@@ -1635,3 +1578,23 @@ unsigned int jit_function_get_max_optimization_level(void)
 	/* TODO - implement more than basic optimization */
 	return 0;
 }
+
+/*@
+ * @deftypefun {jit_label_t} jit_function_reserve_label (jit_function_t @var{func})
+ * Allocate a new label for later use within the function @var{func}. Most
+ * instructions that require a label could perform label allocation themselves.
+ * A separate label allocation could be useful to fill a jump table with
+ * identical entries.
+ * @end deftypefun
+@*/
+jit_label_t
+jit_function_reserve_label(jit_function_t func)
+{
+	/* Ensure that we have a function builder */
+	if(!_jit_function_ensure_builder(func))
+	{
+		return jit_label_undefined;
+	}
+
+	return (func->builder->next_label)++;
+}
