@@ -3,19 +3,21 @@
  *
  * Copyright (C) 2004  Southern Storm Software, Pty Ltd.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This file is part of the libjit library.
  *
- * This program is distributed in the hope that it will be useful,
+ * The libjit library is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation, either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * The libjit library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with the libjit library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "jit-internal.h"
@@ -803,8 +805,14 @@ _jit_gen_load_value(jit_gencode_t gen, int reg, int other_reg, jit_value_t value
 				float32_value = jit_value_get_float32_constant(value);
 				if(IS_WORD_REG(reg))
 				{
-					x86_mov_reg_imm(inst, _jit_reg_info[reg].cpu_reg,
-							((jit_int *)&float32_value)[0]);
+					union
+					{
+						jit_float32 float32_value;
+						jit_int int_value;
+					} un;
+
+					un.float32_value = float32_value;
+					x86_mov_reg_imm(inst, _jit_reg_info[reg].cpu_reg, un.int_value);
 				}
 				else
 				{
@@ -832,10 +840,17 @@ _jit_gen_load_value(jit_gencode_t gen, int reg, int other_reg, jit_value_t value
 				float64_value = jit_value_get_float64_constant(value);
 				if(IS_WORD_REG(reg))
 				{
+					union
+					{
+						jit_float64 float64_value;
+						jit_int int_value[2];
+					} un;
+
+					un.float64_value = float64_value;
 					x86_mov_reg_imm(inst, _jit_reg_info[reg].cpu_reg,
-							((jit_int *)&float64_value)[0]);
+							un.int_value[0]);
 					x86_mov_reg_imm(inst, _jit_reg_info[other_reg].cpu_reg,
-							((jit_int *)&float64_value)[1]);
+							un.int_value[1]);
 				}
 				else
 				{
@@ -863,10 +878,17 @@ _jit_gen_load_value(jit_gencode_t gen, int reg, int other_reg, jit_value_t value
 				nfloat_value = jit_value_get_nfloat_constant(value);
 				if(IS_WORD_REG(reg) && sizeof(jit_nfloat) == sizeof(jit_float64))
 				{
+					union
+					{
+						jit_nfloat nfloat_value;
+						jit_int int_value[2];
+					} un;
+
+					un.nfloat_value = nfloat_value;
 					x86_mov_reg_imm(inst, _jit_reg_info[reg].cpu_reg,
-							((jit_int *)&nfloat_value)[0]);
+							un.int_value[0]);
 					x86_mov_reg_imm(inst, _jit_reg_info[other_reg].cpu_reg,
-							((jit_int *)&nfloat_value)[1]);
+							un.int_value[1]);
 				}
 				else
 				{
