@@ -3,27 +3,38 @@
  *
  * Copyright (C) 2004  Southern Storm Software, Pty Ltd.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This file is part of the libjit library.
  *
- * This program is distributed in the hope that it will be useful,
+ * The libjit library is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation, either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * The libjit library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with the libjit library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include <jit/jit-arch.h>
 #include <jit/jit-defs.h>
-#define	JIT_MEMCPY		"mem_copy"
+
+#if defined(__APPLE__) && defined(__MACH__)
+# define	JIT_MEMCPY	"_mem_copy"
+#else
+# define	JIT_MEMCPY	"mem_copy"
+#endif
+
 #include "jit-apply-func.h"
 #include <stdio.h>
 #include <config.h>
+#if HAVE_STDLIB_H
+	#include <stdlib.h>
+#endif
 #if HAVE_ALLOCA_H
 	#include <alloca.h>
 #endif
@@ -57,13 +68,20 @@ the "jit-apply-rules.h" file.
 	#define	PLATFORM_IS_X86_64	1
 #endif
 
+#if PLATFORM_IS_X86
 /*
  * On x86 the extended precision numbers are 10 bytes long. However certain
  * ABIs define the long double size equal to 12 bytes. The extra 2 bytes are
  * for alignment purposes only and has no significance in computations.
  */
-#if PLATFORM_IS_X86
 #define NFLOAT_SIGNIFICANT_BYTES (sizeof(jit_nfloat) != 12 ? sizeof(jit_nfloat) : 10)
+#elif PLATFORM_IS_X86_64
+/*
+ * On x86_64 the extended precision numbers are 10 bytes long. However certain
+ * ABIs define the long double size equal to 16 bytes. The extra 6 bytes are
+ * for alignment purposes only and has no significance in computations.
+ */
+#define NFLOAT_SIGNIFICANT_BYTES (sizeof(jit_nfloat) != 16 ? sizeof(jit_nfloat) : 10)
 #else
 #define NFLOAT_SIGNIFICANT_BYTES sizeof(jit_nfloat)
 #endif
@@ -83,6 +101,8 @@ the "jit-apply-rules.h" file.
 #ifndef JIT_APPLY_NUM_WORD_REGS
 int num_word_regs = 0;
 int num_float_regs = 0;
+int num_double_regs = 0;
+int num_nfloat_regs = 0;
 int pass_stack_float_as_double = 0;
 int pass_stack_float_as_nfloat = 0;
 int pass_stack_double_as_nfloat = 0;
@@ -96,7 +116,11 @@ int return_float_as_nfloat = 0;
 int return_double_as_nfloat = 0;
 int return_nfloat_as_double = 0;
 int floats_in_word_regs = 0;
+int doubles_in_word_regs = 0;
+int nfloats_in_word_regs = 0;
 int return_floats_after = 0;
+int return_doubles_after = 0;
+int return_nfloats_after = 0;
 int varargs_on_stack = 0;
 int struct_return_special_reg = 0;
 int struct_reg_overlaps_word_reg = 0;
@@ -114,6 +138,8 @@ int pad_float_regs = 0;
 #else
 int num_word_regs = JIT_APPLY_NUM_WORD_REGS;
 int num_float_regs = JIT_APPLY_NUM_FLOAT_REGS;
+int num_double_regs = JIT_APPLY_NUM_DOUBLE_REGS;
+int num_nfloat_regs = JIT_APPLY_NUM_NFLOAT_REGS;
 int pass_stack_float_as_double = JIT_APPLY_PASS_STACK_FLOAT_AS_DOUBLE;
 int pass_stack_float_as_nfloat = JIT_APPLY_PASS_STACK_FLOAT_AS_NFLOAT;
 int pass_stack_double_as_nfloat = JIT_APPLY_PASS_STACK_DOUBLE_AS_NFLOAT;
@@ -127,7 +153,11 @@ int return_float_as_nfloat = JIT_APPLY_RETURN_FLOAT_AS_NFLOAT;
 int return_double_as_nfloat = JIT_APPLY_RETURN_DOUBLE_AS_NFLOAT;
 int return_nfloat_as_double = JIT_APPLY_RETURN_NFLOAT_AS_DOUBLE;
 int floats_in_word_regs = JIT_APPLY_FLOATS_IN_WORD_REGS;
+int doubles_in_word_regs = JIT_APPLY_DOUBLES_IN_WORD_REGS;
+int nfloats_in_word_regs = JIT_APPLY_NFLOATS_IN_WORD_REGS;
 int return_floats_after = JIT_APPLY_RETURN_FLOATS_AFTER;
+int return_doubles_after = JIT_APPLY_RETURN_DOUBLES_AFTER;
+int return_nfloats_after = JIT_APPLY_RETURN_NFLOATS_AFTER;
 int varargs_on_stack = JIT_APPLY_VARARGS_ON_STACK;
 int struct_return_special_reg = JIT_APPLY_STRUCT_RETURN_SPECIAL_REG;
 int struct_reg_overlaps_word_reg = JIT_APPLY_STRUCT_REG_OVERLAPS_WORD_REG;
@@ -318,17 +348,43 @@ struct detect_struct_reg detect_struct_overlap(jit_nint arg1, jit_nint arg2)
 /*
  * Detect the number of floating-point registers.
  */
-void detect_float_regs(double arg1, double arg2, double arg3,
-					   double arg4, double arg5, double arg6,
-					   double arg7, double arg8, double arg9,
-					   double arg10, double arg11, double arg12,
-					   double arg13, double arg14, double arg15,
-					   double arg16, double arg17, double arg18,
-					   double arg19, double arg20, double arg21,
-					   double arg22, double arg23, double arg24,
-					   double arg25, double arg26, double arg27,
-					   double arg28, double arg29, double arg30,
-					   double arg31, double arg32)
+void detect_float_regs(float arg1, float arg2, float arg3,
+					   float arg4, float arg5, float arg6,
+					   float arg7, float arg8, float arg9,
+					   float arg10, float arg11, float arg12,
+					   float arg13, float arg14, float arg15,
+					   float arg16, float arg17, float arg18,
+					   float arg19, float arg20, float arg21,
+					   float arg22, float arg23, float arg24,
+					   float arg25, float arg26, float arg27,
+					   float arg28, float arg29, float arg30,
+					   float arg31, float arg32)
+{
+	jit_nint *args;
+	float *stack_args;
+	jit_builtin_apply_args(jit_nint *, args);
+	stack_args = (float *)(args[0]);
+
+	/* The first stack argument indicates the number of floating-point
+	   registers.  At the moment we don't know if they overlap with
+	   the word registers or not */
+	num_float_regs = (int)(stack_args[0]);
+}
+
+/*
+ * Detect the number of floating-point registers.
+ */
+void detect_double_regs(double arg1, double arg2, double arg3,
+						double arg4, double arg5, double arg6,
+						double arg7, double arg8, double arg9,
+						double arg10, double arg11, double arg12,
+						double arg13, double arg14, double arg15,
+						double arg16, double arg17, double arg18,
+						double arg19, double arg20, double arg21,
+						double arg22, double arg23, double arg24,
+						double arg25, double arg26, double arg27,
+						double arg28, double arg29, double arg30,
+						double arg31, double arg32)
 {
 	jit_nint *args;
 	double *stack_args;
@@ -338,7 +394,33 @@ void detect_float_regs(double arg1, double arg2, double arg3,
 	/* The first stack argument indicates the number of floating-point
 	   registers.  At the moment we don't know if they overlap with
 	   the word registers or not */
-	num_float_regs = (int)(stack_args[0]);
+	num_double_regs = (int)(stack_args[0]);
+}
+
+/*
+ * Detect the number of native floating-point registers.
+ */
+void detect_nfloat_regs(jit_nfloat arg1, jit_nfloat arg2, jit_nfloat arg3,
+						jit_nfloat arg4, jit_nfloat arg5, jit_nfloat arg6,
+						jit_nfloat arg7, jit_nfloat arg8, jit_nfloat arg9,
+						jit_nfloat arg10, jit_nfloat arg11, jit_nfloat arg12,
+						jit_nfloat arg13, jit_nfloat arg14, jit_nfloat arg15,
+						jit_nfloat arg16, jit_nfloat arg17, jit_nfloat arg18,
+						jit_nfloat arg19, jit_nfloat arg20, jit_nfloat arg21,
+						jit_nfloat arg22, jit_nfloat arg23, jit_nfloat arg24,
+						jit_nfloat arg25, jit_nfloat arg26, jit_nfloat arg27,
+						jit_nfloat arg28, jit_nfloat arg29, jit_nfloat arg30,
+						jit_nfloat arg31, jit_nfloat arg32)
+{
+	jit_nint *args;
+	jit_nfloat *stack_args;
+	jit_builtin_apply_args(jit_nint *, args);
+	stack_args = (jit_nfloat *)(args[0]);
+
+	/* The first stack argument indicates the number of floating-point
+	   registers.  At the moment we don't know if they overlap with
+	   the word registers or not */
+	num_nfloat_regs = (int)(stack_args[0]);
 }
 
 #ifdef JIT_NATIVE_INT32
@@ -372,8 +454,25 @@ void detect_double_overlap(double x, jit_nint y, jit_nint z)
 	mem_copy(&temp, args + struct_return_special_reg + 1, sizeof(temp));
 	if(!mem_cmp(&temp, &x, sizeof(double)))
 	{
-		floats_in_word_regs = 1;
-		num_float_regs = 0;
+		doubles_in_word_regs = 1;
+		num_double_regs = 0;
+	}
+}
+
+/*
+ * Detect if a "nfloat" value will use a word register.
+ */
+void detect_nfloat_overlap(jit_nfloat x, jit_nint y, jit_nint z)
+{
+	/* We have an overlap if "x" is in the first word register slot */
+	jit_nfloat temp;
+	jit_nint *args;
+	jit_builtin_apply_args(jit_nint *, args);
+	mem_copy(&temp, args + struct_return_special_reg + 1, sizeof(temp));
+	if(!mem_cmp(&temp, &x, sizeof(double)))
+	{
+		nfloats_in_word_regs = 1;
+		num_nfloat_regs = 0;
 	}
 }
 
@@ -387,7 +486,7 @@ void detect_float_reg_size_regs(double x, double y)
 	jit_builtin_apply_args(jit_nint *, args);
 	mem_copy(&temp, args + 1 + struct_return_special_reg + num_word_regs,
 			 sizeof(temp));
-	if(!mem_cmp(&temp, &x, sizeof(double)))
+	if((num_nfloat_regs > 0) && !mem_cmp(&temp, &x, sizeof(double)))
 	{
 		pass_reg_nfloat_as_double = 1;
 	}
@@ -398,7 +497,10 @@ void detect_float_reg_size_regs(double x, double y)
 				 sizeof(temp));
 		if(!mem_cmp(&temp, &x, sizeof(double)))
 		{
-			pass_reg_nfloat_as_double = 1;
+			if(num_nfloat_regs > 0)
+			{
+				pass_reg_nfloat_as_double = 1;
+			}
 			pad_float_regs = 1;
 		}
 		else
@@ -408,7 +510,10 @@ void detect_float_reg_size_regs(double x, double y)
 					 sizeof(temp));
 			if(!mem_cmp(&temp, &x, sizeof(double)))
 			{
-				pass_reg_nfloat_as_double = 1;
+				if(num_nfloat_regs > 0)
+				{
+					pass_reg_nfloat_as_double = 1;
+				}
 				pad_float_regs = 2;
 			}
 			else
@@ -539,13 +644,16 @@ void detect_float_promotion(float arg1, float arg2, float arg3,
 	if(reg_promote)
 	{
 		/* Promoting "float" to "nfloat" in registers */
-		if(pass_reg_nfloat_as_double)
+		if(num_nfloat_regs > 0)
 		{
-			pass_reg_float_as_double = 1;
-		}
-		else
-		{
-			pass_reg_float_as_nfloat = 1;
+			if(pass_reg_nfloat_as_double)
+			{
+				pass_reg_float_as_double = 1;
+			}
+			else
+			{
+				pass_reg_float_as_nfloat = 1;
+			}
 		}
 	}
 	if(stack_promote == 2)
@@ -594,7 +702,7 @@ void detect_double_promotion(double arg1, double arg2, double arg3,
 
 	/* Skip the arguments that will be stored in registers */
 	index = 1;
-	if(floats_in_word_regs)
+	if(doubles_in_word_regs)
 	{
 		if(sizeof(jit_nint) == sizeof(jit_int))
 		{
@@ -701,9 +809,9 @@ void detect_float_return(void)
 	float float_value;
 	double double_value;
 	jit_nfloat nfloat_value;
-	int float_size;
-	int double_size;
-	int nfloat_size;
+	int float_size = 0;
+	int double_size = 0;
+	int nfloat_size = 0;
 	float temp_float;
 	double temp_double;
 	jit_nfloat temp_nfloat;
@@ -715,6 +823,7 @@ void detect_float_return(void)
 	jit_builtin_apply(return_float, args, 0, 1, return_value);
 
 	/* Find the location of the return value */
+	/* and determine the size of the "float" return value */
 	offset = 0;
 	while(offset < 64)
 	{
@@ -731,72 +840,24 @@ void detect_float_return(void)
 				temp_nfloat = (jit_nfloat)123.0;
 				if(!mem_cmp(&nfloat_value, &temp_nfloat, NFLOAT_SIGNIFICANT_BYTES))
 				{
+					float_size = 3;
 					break;
 				}
 			}
 			else
 			{
+				float_size = 2;
 				break;
 			}
 		}
 		else
 		{
+			float_size = 1;
 			break;
 		}
 		offset += sizeof(void *);
 	}
 
-	/* Determine the size of the "float" return value */
-	mem_copy(&float_value, return_value + offset, sizeof(float));
-	temp_float = (float)123.0;
-	if(!mem_cmp(&float_value, &temp_float, sizeof(float)))
-	{
-		float_size = 1;
-	}
-	else
-	{
-		mem_copy(&double_value, return_value + offset, sizeof(double));
-		temp_double = (double)123.0;
-		if(!mem_cmp(&double_value, &temp_double, sizeof(double)))
-		{
-			float_size = 2;
-		}
-		else
-		{
-			float_size = 3;
-		}
-	}
-
-	/* Call "return_double" and get its return structure */
-	jit_builtin_apply(return_double, args, 0, 1, return_value);
-
-	/* Determine the size of the "double" return value */
-	mem_copy(&double_value, return_value + offset, sizeof(double));
-	temp_double = (double)456.7;
-	if(!mem_cmp(&double_value, &temp_double, sizeof(double)))
-	{
-		double_size = 2;
-	}
-	else
-	{
-		double_size = 3;
-	}
-
-	/* Call "return_nfloat" and get its return structure */
-	jit_builtin_apply(return_nfloat, args, 0, 1, return_value);
-
-	/* Determine the size of the "nfloat" return value */
-	mem_copy(&double_value, return_value + offset, sizeof(double));
-	temp_double = (double)8901.2;
-	if(!mem_cmp(&double_value, &temp_double, sizeof(double)))
-	{
-		nfloat_size = 2;
-	}
-	else
-	{
-		nfloat_size = 3;
-	}
-
 	/* Use the offset and size information to set the final parameters */
 	return_floats_after = offset;
 	if(float_size == 2)
@@ -807,10 +868,74 @@ void detect_float_return(void)
 	{
 		return_float_as_nfloat = 1;
 	}
+
+	/* Call "return_double" and get its return structure */
+	jit_builtin_apply(return_double, args, 0, 1, return_value);
+
+	/* Find the location of the return value */
+	/* and determine the size of the "double" return value */
+	offset = 0;
+	while(offset < 64)
+	{
+		mem_copy(&double_value, return_value + offset, sizeof(double));
+		temp_double = (double)456.7;
+		if(mem_cmp(&double_value, &temp_double, sizeof(double)))
+		{
+			mem_copy(&nfloat_value, return_value + offset,
+					 sizeof(jit_nfloat));
+			temp_nfloat = (jit_nfloat)456.7;
+			if(!mem_cmp(&nfloat_value, &temp_nfloat, NFLOAT_SIGNIFICANT_BYTES))
+			{
+				double_size = 3;
+				break;
+			}
+		}
+		else
+		{
+			double_size = 2;
+			break;
+		}
+		offset += sizeof(void *);
+	}
+
+	/* Use the offset and size information to set the final parameters */
+	return_doubles_after = offset;
 	if(double_size == 3)
 	{
 		return_double_as_nfloat = 1;
 	}
+
+	/* Call "return_nfloat" and get its return structure */
+	jit_builtin_apply(return_nfloat, args, 0, 1, return_value);
+
+	/* Find the location of the return value */
+	/* and determine the size of the "nfloat" return value */
+	offset = 0;
+	while(offset < 64)
+	{
+		mem_copy(&double_value, return_value + offset, sizeof(double));
+		temp_double = (double)8901.2;
+		if(mem_cmp(&double_value, &temp_double, sizeof(double)))
+		{
+			mem_copy(&nfloat_value, return_value + offset,
+					 sizeof(jit_nfloat));
+			temp_nfloat = (jit_nfloat)8901.2;
+			if(!mem_cmp(&nfloat_value, &temp_nfloat, NFLOAT_SIGNIFICANT_BYTES))
+			{
+				nfloat_size = 3;
+				break;
+			}
+		}
+		else
+		{
+			nfloat_size = 2;
+			break;
+		}
+		offset += sizeof(void *);
+	}
+
+	/* Use the offset and size information to set the final parameters */
+	return_nfloats_after = offset;
 	if(nfloat_size == 2)
 	{
 		return_nfloat_as_double = 1;
@@ -1399,8 +1524,7 @@ void dump_return_union(void)
 	const char *double_type;
 	const char *nfloat_type;
 
-	/* Dump the definition of "jit_apply_float" */
-	printf("typedef union\n{\n");
+	/* Determine the float return types */
 	if(return_float_as_nfloat)
 	{
 		float_type = "jit_nfloat";
@@ -1429,10 +1553,6 @@ void dump_return_union(void)
 	{
 		nfloat_type = "jit_nfloat";
 	}
-	printf("\t%s float_value;\n", float_type);
-	printf("\t%s double_value;\n", double_type);
-	printf("\t%s nfloat_value;\n", nfloat_type);
-	printf("\n} jit_apply_float;\n");
 
 	/* Dump the definition of "jit_apply_return" */
 	printf("typedef union\n{\n");
@@ -1442,12 +1562,30 @@ void dump_return_union(void)
 	printf("\tjit_ulong ulong_value;\n");
 	if(return_floats_after)
 	{
-		printf("\tstruct { jit_ubyte pad[%d]; jit_apply_float inner_value; } f_value;\n",
-			   return_floats_after);
+		printf("\tstruct { jit_ubyte pad[%d]; %s f_value; } float_value;\n",
+			   return_floats_after, float_type);
 	}
 	else
 	{
-		printf("\tstruct { jit_apply_float inner_value; } f_value;\n");
+		printf("\tstruct { %s f_value; } float_value;\n", float_type);
+	}
+	if(return_doubles_after)
+	{
+		printf("\tstruct { jit_ubyte pad[%d]; %s f_value; } double_value;\n",
+			   return_doubles_after, double_type);
+	}
+	else
+	{
+		printf("\tstruct { %s f_value; } double_value;\n", double_type);
+	}
+	if(return_nfloats_after)
+	{
+		printf("\tstruct { jit_ubyte pad[%d]; %s f_value; } nfloat_value;\n",
+			   return_nfloats_after, nfloat_type);
+	}
+	else
+	{
+		printf("\tstruct { %s f_value; } nfloat_value;\n", nfloat_type);
 	}
 	if(max_struct_in_reg > 0)
 	{
@@ -1477,11 +1615,11 @@ void dump_return_union(void)
 	printf("#define jit_apply_return_get_ulong(result)\t\\\n");
 	printf("\t((jit_ulong)((result)->ulong_value))\n");
 	printf("#define jit_apply_return_get_float32(result)\t\\\n");
-	printf("\t((jit_float32)((result)->f_value.inner_value.float_value))\n");
+	printf("\t((jit_float32)((result)->float_value.f_value))\n");
 	printf("#define jit_apply_return_get_float64(result)\t\\\n");
-	printf("\t((jit_float64)((result)->f_value.inner_value.double_value))\n");
+	printf("\t((jit_float64)((result)->double_value.f_value))\n");
 	printf("#define jit_apply_return_get_nfloat(result)\t\\\n");
-	printf("\t((jit_nfloat)((result)->f_value.inner_value.nfloat_value))\n");
+	printf("\t((jit_nfloat)((result)->nfloat_value.f_value))\n");
 	printf("\n");
 	printf("#define jit_apply_return_set_sbyte(result,value)\t\\\n");
 	printf("\t(((result)->int_value) = ((jit_nint)(value)))\n");
@@ -1504,13 +1642,13 @@ void dump_return_union(void)
 	printf("#define jit_apply_return_set_ulong(result,value)\t\\\n");
 	printf("\t(((result)->ulong_value) = ((jit_ulong)(value)))\n");
 	printf("#define jit_apply_return_set_float32(result,value)\t\\\n");
-	printf("\t(((result)->f_value.inner_value.float_value) = ((%s)(value)))\n",
+	printf("\t(((result)->float_value.f_value) = ((%s)(value)))\n",
 		   float_type);
 	printf("#define jit_apply_return_set_float64(result,value)\t\\\n");
-	printf("\t(((result)->f_value.inner_value.double_value) = ((%s)(value)))\n",
+	printf("\t(((result)->double_value.f_value) = ((%s)(value)))\n",
 		   double_type);
 	printf("#define jit_apply_return_set_nfloat(result,value)\t\\\n");
-	printf("\t(((result)->f_value.inner_value.nfloat_value) = ((%s)(value)))\n",
+	printf("\t(((result)->nfloat_value.f_value) = ((%s)(value)))\n",
 		   nfloat_type);
 	printf("\n");
 }
@@ -1521,14 +1659,70 @@ void dump_return_union(void)
 void dump_apply_structure(void)
 {
 	const char *name;
-	if(pass_reg_nfloat_as_double)
-		name = "jit_float64";
-	else
-		name = "jit_nfloat";
 	if(num_float_regs > 0)
 	{
+		if(pass_reg_float_as_double)
+		{
+			name = "jit_float64";
+		}
+		else if(pass_reg_float_as_nfloat)
+		{
+			name = "jit_nfloat";
+		}	
+		else
+		{
+			name = "jit_float32";
+		}
 		printf("typedef %s jit_reg_float;\n\n", name);
 	}
+	if(num_double_regs > 0)
+	{
+		if(pass_reg_double_as_nfloat)
+		{
+			name = "jit_nfloat";
+		}
+		else
+		{
+			name = "jit_float64";
+		}
+		printf("typedef %s jit_reg_double;\n\n", name);
+	}
+	if(num_nfloat_regs > 0)
+	{
+		if(pass_reg_nfloat_as_double)
+		{
+			name = "jit_float64";
+		}
+		else
+		{
+			name = "jit_nfloat";
+		}
+		printf("typedef %s jit_reg_nfloat;\n\n", name);
+	}
+	if((num_float_regs > 0) ||
+	   (num_double_regs > 0) ||
+	   (num_nfloat_regs > 0))
+	{
+		printf("typedef union\n{\n");
+		if(num_float_regs > 0)
+		{
+			printf("\tjit_reg_float float_value;\n");
+		}
+		if(num_double_regs > 0)
+		{
+			printf("\tjit_reg_double double_value;\n");
+		}
+		if(num_nfloat_regs > 0)
+		{
+			printf("\tjit_reg_nfloat nfloat_value;\n");
+		}
+		if(pad_float_regs > 0)
+		{
+			printf("\tchar __pad[%i];\n",
+				   (int)(sizeof(double) + pad_float_regs * sizeof(jit_nint)));
+		}
+		printf("} jit_reg_float_struct;\n\n");
+	}
 	printf("typedef struct\n{\n");
 	printf("\tunsigned char *stack_args;\n");
 	if(struct_return_special_reg)
@@ -1547,9 +1741,11 @@ void dump_apply_structure(void)
 	{
 		printf("\tjit_nint pad[%d];\n", pad_float_regs);
 	}
-	if(num_float_regs > 0)
+	if((num_float_regs > 0) ||
+	   (num_double_regs > 0) ||
+	   (num_nfloat_regs > 0))
 	{
-		printf("\tjit_reg_float float_regs[%d];\n", num_float_regs);
+		printf("\tjit_reg_float_struct float_regs[%d];\n", num_float_regs);
 	}
 	printf("\n} jit_apply_struct;\n\n");
 }
@@ -1583,6 +1779,15 @@ void dump_apply_macros(void)
 	printf("\tvoid *struct_return;\n");
 	printf("\n} jit_apply_builder;\n\n");
 
+	/* Include jit-apply-func.h here to allow the backend to add it's definitions */
+	printf("#include \"jit-apply-func.h\"\n\n");
+
+	printf("void\n_jit_builtin_apply_add_struct(jit_apply_builder *builder, void *value, jit_type_t struct_type);\n\n");
+
+	printf("void\n_jit_builtin_apply_get_struct(jit_apply_builder *builder, void *value, jit_type_t struct_type);\n\n");
+
+	printf("void\n_jit_builtin_apply_get_struct_return(jit_apply_builder *builder, void *return_value, jit_apply_return *apply_return, jit_type_t struct_type);\n\n");
+
 	/* Macro to initialize the apply builder */
 	printf("#define jit_apply_builder_init(builder,type)\t\\\n");
 	printf("\tdo { \\\n");
@@ -1957,7 +2162,7 @@ void dump_apply_macros(void)
 		printf("\tdo { \\\n");
 		printf("\t\tif((builder)->float_used < %d) \\\n", num_float_regs);
 		printf("\t\t{ \\\n");
-		printf("\t\t\t(builder)->apply_args->float_regs[(builder)->float_used] = (jit_reg_float)(value); \\\n");
+		printf("\t\t\t(builder)->apply_args->float_regs[(builder)->float_used].float_value = (jit_reg_float)(value); \\\n");
 		printf("\t\t\t++((builder)->float_used); \\\n");
 		printf("\t\t} \\\n");
 		printf("\t\telse \\\n");
@@ -1973,44 +2178,6 @@ void dump_apply_macros(void)
 		printf("\t\t\t(builder)->stack_used += (sizeof(%s) + sizeof(jit_nint) - 1) & ~(sizeof(jit_nint) - 1); \\\n", name);
 		printf("\t\t} \\\n");
 		printf("\t} while (0)\n");
-
-		printf("#define jit_apply_builder_add_float64(builder,value) \\\n");
-		printf("\tdo { \\\n");
-		printf("\t\tif((builder)->float_used < %d) \\\n", num_float_regs);
-		printf("\t\t{ \\\n");
-		printf("\t\t\t(builder)->apply_args->float_regs[(builder)->float_used] = (jit_reg_float)(value); \\\n");
-		printf("\t\t\t++((builder)->float_used); \\\n");
-		printf("\t\t} \\\n");
-		printf("\t\telse \\\n");
-		if(pass_stack_double_as_nfloat)
-			name = "jit_nfloat";
-		else
-			name = "jit_float64";
-		printf("\t\t{ \\\n");
-		printf("\t\t\t%s __temp = (%s)(value); \\\n", name, name);
-		printf("\t\t\tjit_memcpy((builder)->apply_args->stack_args + (builder)->stack_used, &__temp, sizeof(__temp)); \\\n");
-		printf("\t\t\t(builder)->stack_used += (sizeof(%s) + sizeof(jit_nint) - 1) & ~(sizeof(jit_nint) - 1); \\\n", name);
-		printf("\t\t} \\\n");
-		printf("\t} while (0)\n");
-
-		printf("#define jit_apply_builder_add_nfloat(builder,value) \\\n");
-		printf("\tdo { \\\n");
-		printf("\t\tif((builder)->float_used < %d) \\\n", num_float_regs);
-		printf("\t\t{ \\\n");
-		printf("\t\t\t(builder)->apply_args->float_regs[(builder)->float_used] = (jit_reg_float)(value); \\\n");
-		printf("\t\t\t++((builder)->float_used); \\\n");
-		printf("\t\t} \\\n");
-		printf("\t\telse \\\n");
-		if(pass_stack_nfloat_as_double)
-			name = "jit_float64";
-		else
-			name = "jit_nfloat";
-		printf("\t\t{ \\\n");
-		printf("\t\t\t%s __temp = (%s)(value); \\\n", name, name);
-		printf("\t\t\tjit_memcpy((builder)->apply_args->stack_args + (builder)->stack_used, &__temp, sizeof(__temp)); \\\n");
-		printf("\t\t\t(builder)->stack_used += (sizeof(%s) + sizeof(jit_nint) - 1) & ~(sizeof(jit_nint) - 1); \\\n", name);
-		printf("\t\t} \\\n");
-		printf("\t} while (0)\n");
 	}
 	else if(floats_in_word_regs)
 	{
@@ -2023,20 +2190,6 @@ void dump_apply_macros(void)
 			name = "jit_float32";
 		printf("#define jit_apply_builder_add_float32(builder,value) \\\n");
 		printf("\tjit_apply_builder_add_large((builder), %s, (value));\n", name);
-
-		if(pass_reg_double_as_nfloat)
-			name = "jit_nfloat";
-		else
-			name = "jit_float64";
-		printf("#define jit_apply_builder_add_float64(builder,value) \\\n");
-		printf("\tjit_apply_builder_add_large((builder), %s, (value));\n", name);
-
-		if(pass_reg_nfloat_as_double)
-			name = "jit_float64";
-		else
-			name = "jit_nfloat";
-		printf("#define jit_apply_builder_add_nfloat(builder,value) \\\n");
-		printf("\tjit_apply_builder_add_large((builder), %s, (value));\n", name);
 	}
 	else
 	{
@@ -2049,14 +2202,82 @@ void dump_apply_macros(void)
 			name = "jit_float32";
 		printf("#define jit_apply_builder_add_float32(builder,value) \\\n");
 		printf("\tjit_apply_builder_add_large_stack((builder), %s, (value));\n", name);
-
+	}
+	if(num_double_regs > 0)
+	{
+		printf("#define jit_apply_builder_add_float64(builder,value) \\\n");
+		printf("\tdo { \\\n");
+		printf("\t\tif((builder)->float_used < %d) \\\n", num_double_regs);
+		printf("\t\t{ \\\n");
+		printf("\t\t\t(builder)->apply_args->float_regs[(builder)->float_used].double_value = (jit_reg_double)(value); \\\n");
+		printf("\t\t\t++((builder)->float_used); \\\n");
+		printf("\t\t} \\\n");
+		printf("\t\telse \\\n");
+		if(pass_stack_double_as_nfloat)
+			name = "jit_nfloat";
+		else
+			name = "jit_float64";
+		printf("\t\t{ \\\n");
+		printf("\t\t\t%s __temp = (%s)(value); \\\n", name, name);
+		printf("\t\t\tjit_memcpy((builder)->apply_args->stack_args + (builder)->stack_used, &__temp, sizeof(__temp)); \\\n");
+		printf("\t\t\t(builder)->stack_used += (sizeof(%s) + sizeof(jit_nint) - 1) & ~(sizeof(jit_nint) - 1); \\\n", name);
+		printf("\t\t} \\\n");
+		printf("\t} while (0)\n");
+	}
+	else if(doubles_in_word_regs)
+	{
+		/* Pass double values in word registers */
+		if(pass_reg_double_as_nfloat)
+			name = "jit_nfloat";
+		else
+			name = "jit_float64";
+		printf("#define jit_apply_builder_add_float64(builder,value) \\\n");
+		printf("\tjit_apply_builder_add_large((builder), %s, (value));\n", name);
+	}
+	else
+	{
+		/* Pass double values on the stack */
 		if(pass_stack_double_as_nfloat)
 			name = "jit_nfloat";
 		else
 			name = "jit_float64";
 		printf("#define jit_apply_builder_add_float64(builder,value) \\\n");
 		printf("\tjit_apply_builder_add_large_stack((builder), %s, (value));\n", name);
-
+	}
+	if(num_nfloat_regs > 0)
+	{
+		printf("#define jit_apply_builder_add_nfloat(builder,value) \\\n");
+		printf("\tdo { \\\n");
+		printf("\t\tif((builder)->float_used < %d) \\\n", num_nfloat_regs);
+		printf("\t\t{ \\\n");
+		printf("\t\t\t(builder)->apply_args->float_regs[(builder)->float_used].nfloat_value = (jit_reg_nfloat)(value); \\\n");
+		printf("\t\t\t++((builder)->float_used); \\\n");
+		printf("\t\t} \\\n");
+		printf("\t\telse \\\n");
+		if(pass_stack_nfloat_as_double)
+			name = "jit_float64";
+		else
+			name = "jit_nfloat";
+		printf("\t\t{ \\\n");
+		printf("\t\t\t%s __temp = (%s)(value); \\\n", name, name);
+		printf("\t\t\tjit_memcpy((builder)->apply_args->stack_args + (builder)->stack_used, &__temp, sizeof(__temp)); \\\n");
+		printf("\t\t\t(builder)->stack_used += (sizeof(%s) + sizeof(jit_nint) - 1) & ~(sizeof(jit_nint) - 1); \\\n", name);
+		printf("\t\t} \\\n");
+		printf("\t} while (0)\n");
+	}
+	else if(nfloats_in_word_regs)
+	{
+		/* Pass nfloat values in word registers */
+		if(pass_reg_nfloat_as_double)
+			name = "jit_float64";
+		else
+			name = "jit_nfloat";
+		printf("#define jit_apply_builder_add_nfloat(builder,value) \\\n");
+		printf("\tjit_apply_builder_add_large((builder), %s, (value));\n", name);
+	}
+	else
+	{
+		/* Pass nfloat values on the stack */
 		if(pass_stack_nfloat_as_double)
 			name = "jit_float64";
 		else
@@ -2103,12 +2324,12 @@ void dump_apply_macros(void)
 #endif
 	if(num_float_regs > 0)
 	{
-		/* Pass floating point values in registers, if possible */
+		/* Pass float values in registers, if possible */
 		printf("#define jit_apply_parser_get_float32(builder,value) \\\n");
 		printf("\tdo { \\\n");
 		printf("\t\tif((builder)->float_used < %d) \\\n", num_float_regs);
 		printf("\t\t{ \\\n");
-		printf("\t\t\t(value) = (jit_float32)((builder)->apply_args->float_regs[(builder)->float_used]); \\\n");
+		printf("\t\t\t(value) = (jit_float32)((builder)->apply_args->float_regs[(builder)->float_used].float_value); \\\n");
 		printf("\t\t\t++((builder)->float_used); \\\n");
 		printf("\t\t} \\\n");
 		printf("\t\telse \\\n");
@@ -2125,12 +2346,39 @@ void dump_apply_macros(void)
 		printf("\t\t\t(value) = (jit_float32)__temp; \\\n");
 		printf("\t\t} \\\n");
 		printf("\t} while (0)\n");
-
+	}
+	else if(floats_in_word_regs)
+	{
+		/* Pass float values in word registers */
+		if(pass_reg_float_as_double)
+			name = "jit_float64";
+		else if(pass_reg_float_as_nfloat)
+			name = "jit_nfloat";
+		else
+			name = "jit_float32";
+		printf("#define jit_apply_parser_get_float32(builder,value) \\\n");
+		printf("\tjit_apply_parser_get_large((builder), %s, jit_float32, (value));\n", name);
+	}
+	else
+	{
+		/* Pass float values on the stack */
+		if(pass_stack_float_as_double)
+			name = "jit_float64";
+		else if(pass_stack_float_as_nfloat)
+			name = "jit_nfloat";
+		else
+			name = "jit_float32";
+		printf("#define jit_apply_parser_get_float32(builder,value) \\\n");
+		printf("\tjit_apply_parser_get_large_stack((builder), %s, jit_float32, (value));\n", name);
+	}
+	if(num_double_regs > 0)
+	{
+		/* Pass double values in registers, if possible */
 		printf("#define jit_apply_parser_get_float64(builder,value) \\\n");
 		printf("\tdo { \\\n");
-		printf("\t\tif((builder)->float_used < %d) \\\n", num_float_regs);
+		printf("\t\tif((builder)->float_used < %d) \\\n", num_double_regs);
 		printf("\t\t{ \\\n");
-		printf("\t\t\t(value) = (jit_float64)((builder)->apply_args->float_regs[(builder)->float_used]); \\\n");
+		printf("\t\t\t(value) = (jit_float64)((builder)->apply_args->float_regs[(builder)->float_used].double_value); \\\n");
 		printf("\t\t\t++((builder)->float_used); \\\n");
 		printf("\t\t} \\\n");
 		printf("\t\telse \\\n");
@@ -2145,12 +2393,35 @@ void dump_apply_macros(void)
 		printf("\t\t\t(value) = (jit_float64)__temp; \\\n");
 		printf("\t\t} \\\n");
 		printf("\t} while (0)\n");
-
+	}
+	else if(doubles_in_word_regs)
+	{
+		/* Pass double values in word registers */
+		if(pass_reg_double_as_nfloat)
+			name = "jit_nfloat";
+		else
+			name = "jit_float64";
+		printf("#define jit_apply_parser_get_float64(builder,value) \\\n");
+		printf("\tjit_apply_parser_get_large((builder), %s, jit_float64, (value));\n", name);
+	}
+	else
+	{
+		/* Pass double values on the stack */
+		/* Pass nfloat values in registers, if possible */
+		if(pass_stack_double_as_nfloat)
+			name = "jit_nfloat";
+		else
+			name = "jit_float64";
+		printf("#define jit_apply_parser_get_float64(builder,value) \\\n");
+		printf("\tjit_apply_parser_get_large_stack((builder), %s, jit_float64, (value));\n", name);
+	}
+	if(num_nfloat_regs > 0)
+	{
 		printf("#define jit_apply_parser_get_nfloat(builder,value) \\\n");
 		printf("\tdo { \\\n");
-		printf("\t\tif((builder)->float_used < %d) \\\n", num_float_regs);
+		printf("\t\tif((builder)->float_used < %d) \\\n", num_nfloat_regs);
 		printf("\t\t{ \\\n");
-		printf("\t\t\t(value) = (jit_nfloat)((builder)->apply_args->float_regs[(builder)->float_used]); \\\n");
+		printf("\t\t\t(value) = (jit_nfloat)((builder)->apply_args->float_regs[(builder)->float_used].nfloat_value); \\\n");
 		printf("\t\t\t++((builder)->float_used); \\\n");
 		printf("\t\t} \\\n");
 		printf("\t\telse \\\n");
@@ -2166,25 +2437,9 @@ void dump_apply_macros(void)
 		printf("\t\t} \\\n");
 		printf("\t} while (0)\n");
 	}
-	else if(floats_in_word_regs)
+	else if(nfloats_in_word_regs)
 	{
-		/* Pass floating point values in word registers */
-		if(pass_reg_float_as_double)
-			name = "jit_float64";
-		else if(pass_reg_float_as_nfloat)
-			name = "jit_nfloat";
-		else
-			name = "jit_float32";
-		printf("#define jit_apply_parser_get_float32(builder,value) \\\n");
-		printf("\tjit_apply_parser_get_large((builder), %s, jit_float32, (value));\n", name);
-
-		if(pass_reg_double_as_nfloat)
-			name = "jit_nfloat";
-		else
-			name = "jit_float64";
-		printf("#define jit_apply_parser_get_float64(builder,value) \\\n");
-		printf("\tjit_apply_parser_get_large((builder), %s, jit_float64, (value));\n", name);
-
+		/* Pass nfloat values in word registers */
 		if(pass_reg_nfloat_as_double)
 			name = "jit_float64";
 		else
@@ -2194,23 +2449,7 @@ void dump_apply_macros(void)
 	}
 	else
 	{
-		/* Pass floating point values on the stack */
-		if(pass_stack_float_as_double)
-			name = "jit_float64";
-		else if(pass_stack_float_as_nfloat)
-			name = "jit_nfloat";
-		else
-			name = "jit_float32";
-		printf("#define jit_apply_parser_get_float32(builder,value) \\\n");
-		printf("\tjit_apply_parser_get_large_stack((builder), %s, jit_float32, (value));\n", name);
-
-		if(pass_stack_double_as_nfloat)
-			name = "jit_nfloat";
-		else
-			name = "jit_float64";
-		printf("#define jit_apply_parser_get_float64(builder,value) \\\n");
-		printf("\tjit_apply_parser_get_large_stack((builder), %s, jit_float64, (value));\n", name);
-
+		/* Pass nfloat values on the stack */
 		if(pass_stack_nfloat_as_double)
 			name = "jit_float64";
 		else
@@ -2277,12 +2516,24 @@ int main(int argc, char *argv[])
 	/* Determine if the special structure register overlaps a word register */
 	detect_struct_buf = detect_struct_overlap(1, 2);
 
-	/* Detect the number of floating-point registers */
+	/* Detect the number of float registers */
 	detect_float_regs(0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0,
 					  9.0, 10.0, 11.0, 12.0, 13.0, 14.0, 15.0, 16.0,
 					  17.0, 18.0, 19.0, 20.0, 21.0, 22.0, 23.0, 24.0,
 					  25.0, 26.0, 27.0, 28.0, 29.0, 30.0, 31.0);
 
+	/* Detect the number of double registers */
+	detect_double_regs(0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0,
+					   9.0, 10.0, 11.0, 12.0, 13.0, 14.0, 15.0, 16.0,
+					   17.0, 18.0, 19.0, 20.0, 21.0, 22.0, 23.0, 24.0,
+					   25.0, 26.0, 27.0, 28.0, 29.0, 30.0, 31.0);
+
+	/* Detect the number of native floating-point registers */
+	detect_nfloat_regs(0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0,
+					  9.0, 10.0, 11.0, 12.0, 13.0, 14.0, 15.0, 16.0,
+					  17.0, 18.0, 19.0, 20.0, 21.0, 22.0, 23.0, 24.0,
+					  25.0, 26.0, 27.0, 28.0, 29.0, 30.0, 31.0);
+
 	/* Determine if floating-point values are passed in word registers */
 	if(num_float_regs > 0 && num_word_regs > 0)
 	{
@@ -2298,6 +2549,11 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	if(num_nfloat_regs > 0 && num_word_regs > 0)
+	{
+		detect_nfloat_overlap(123.78, 1, 2);
+	}
+
 	/* Determine if "long double" values should be demoted to "double" */
 	if(floats_in_word_regs)
 	{
@@ -2369,6 +2625,8 @@ int main(int argc, char *argv[])
 	printf("#define _JIT_APPLY_RULES_H\n\n");
 	printf("#define JIT_APPLY_NUM_WORD_REGS %d\n", num_word_regs);
 	printf("#define JIT_APPLY_NUM_FLOAT_REGS %d\n", num_float_regs);
+	printf("#define JIT_APPLY_NUM_DOUBLE_REGS %d\n", num_double_regs);
+	printf("#define JIT_APPLY_NUM_NFLOAT_REGS %d\n", num_nfloat_regs);
 	printf("#define JIT_APPLY_PASS_STACK_FLOAT_AS_DOUBLE %d\n",
 		   pass_stack_float_as_double);
 	printf("#define JIT_APPLY_PASS_STACK_FLOAT_AS_NFLOAT %d\n",
@@ -2390,7 +2648,11 @@ int main(int argc, char *argv[])
 	printf("#define JIT_APPLY_RETURN_DOUBLE_AS_NFLOAT %d\n", return_double_as_nfloat);
 	printf("#define JIT_APPLY_RETURN_NFLOAT_AS_DOUBLE %d\n", return_nfloat_as_double);
 	printf("#define JIT_APPLY_FLOATS_IN_WORD_REGS %d\n", floats_in_word_regs);
+	printf("#define JIT_APPLY_DOUBLES_IN_WORD_REGS %d\n", doubles_in_word_regs);
+	printf("#define JIT_APPLY_NFLOATS_IN_WORD_REGS %d\n", nfloats_in_word_regs);
 	printf("#define JIT_APPLY_RETURN_FLOATS_AFTER %d\n", return_floats_after);
+	printf("#define JIT_APPLY_RETURN_DOUBLES_AFTER %d\n", return_doubles_after);
+	printf("#define JIT_APPLY_RETURN_NFLOATS_AFTER %d\n", return_nfloats_after);
 	printf("#define JIT_APPLY_VARARGS_ON_STACK %d\n", varargs_on_stack);
 	printf("#define JIT_APPLY_STRUCT_RETURN_SPECIAL_REG %d\n", struct_return_special_reg);
 	printf("#define JIT_APPLY_STRUCT_REG_OVERLAPS_WORD_REG %d\n",
